/************************************************
 *
 *  CROSSCONTROL AUX API
 *
 *  AuxVersion.h
 *
 *  Version 2.15.2.0, 2020-07-08
 *
 *  (c) CrossControl 2010-2020
 *

 *  File description:
 *  Get software versions for firmware and software
 *
 *  Use the function GetAuxVersion() to get a handle to pass on to the other functions. 
 *  Use the method AuxVersion_release(AUXVERSIONHANDLE) to return the handle.
 *
 *  Example Usage:
 *  @include auxversion_example.cpp
 *
 ************************************************/

#ifndef CC_VERSION_H_
#define CC_VERSION_H_

#include "CCAuxErrors.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* AUXVERSIONHANDLE;

  /*!
   * Factory function that creates instances of the AuxVersion object.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @return AUXVERSIONHANDLE to an allocated AuxVersion object. The returned
   * handle needs to be deallocated using the AuxVersion_release(AUXVERSIONHANDLE)
   * method when it's no longer needed.
   * Returns NULL if it fails to allocate memory.
   *
   * Example Usage:
   * @snippet auxversion_example.cpp Calling GetAuxVersion
   */
  EXTERN_C
    CCAUXDLL_API
    AUXVERSIONHANDLE
    CCAUXDLL_CALLING_CONV
    GetAuxVersion(void);

  /*!
  * Delete the AuxVersion object.
  *
  * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
  *
  * @return -
  *
  * Example Usage:
  * @snippet auxversion_example.cpp Calling AuxVersion_release
  */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    AuxVersion_release(AUXVERSIONHANDLE);

  /*!
  * Get the FPGA software version
  *
  * Supported Platform(s): XL, XM, XS, XA, VS
  *
  * @param major Major version number
  * @param minor Minor version  number
  * @param release Release version number
  * @param build Build version number
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet auxversion_example.cpp Calling AuxVersion_getFPGAVersion
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    AuxVersion_getFPGAVersion(
    AUXVERSIONHANDLE,
    uint8_t* major,
    uint8_t* minor,
    uint8_t* release,
    uint8_t* build);

  /*!
  * Get the System Supervisor software version
  *
  * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
  *
  * @param major Major version number
  * @param minor Minor version  number
  * @param release Release version number
  * @param build Build version number
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet auxversion_example.cpp Calling AuxVersion_getSSVersion
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    AuxVersion_getSSVersion(
    AUXVERSIONHANDLE,
    uint8_t* major,
    uint8_t* minor,
    uint8_t* release,
    uint8_t* build);

  /*!
  * Get the front microcontroller software version
  *
  * Supported Platform(s): XL, XM, XS, XA
  *
  * @param major Major version number
  * @param minor Minor version  number
  * @param release Release version number
  * @param build Build version number
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet auxversion_example.cpp Calling AuxVersion_getFrontVersion
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    AuxVersion_getFrontVersion(
    AUXVERSIONHANDLE,
    uint8_t* major,
    uint8_t* minor,
    uint8_t* release,
    uint8_t* build);

  /*!
  * Get the CrossControl CCAux API version. CCAux includes:
  *   CCAuxService/ccauxd - Windows Service/Linux daemon.
  *   CCAux2.dll/libccaux2 - The implementation of this API.
  *
  * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
  *
  * @param major Major version number
  * @param minor Minor version  number
  * @param release Release version number
  * @param build Build version number
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet auxversion_example.cpp Calling AuxVersion_getCCAuxVersion
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    AuxVersion_getCCAuxVersion(
    AUXVERSIONHANDLE,
    uint8_t* major,
    uint8_t* minor,
    uint8_t* release,
    uint8_t* build);

  /*!
  * Get the CrossControl Operating System version.
  *
  * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
  *
  * @param major Major version number
  * @param minor Minor version  number
  * @param release Release version number
  * @param build Build version number
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet auxversion_example.cpp Calling AuxVersion_getOSVersion
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    AuxVersion_getOSVersion(
    AUXVERSIONHANDLE,
    uint8_t* major, 
    uint8_t* minor, 
    uint8_t* release, 
    uint8_t* build);

  /*!
  * Get the CrossControl CCAux CCAuxDrv version.
  * Can be used to check that the correct driver is loaded.
  *
  * Supported Platform(s): XL, XM
  *
  * @param major Major version number
  * @param minor Minor version  number
  * @param release Release version number
  * @param build Build version number
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet auxversion_example.cpp Calling AuxVersion_getCCAuxDrvVersion
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    AuxVersion_getCCAuxDrvVersion(
    AUXVERSIONHANDLE,
    uint8_t* major, 
    uint8_t* minor, 
    uint8_t* release, 
    uint8_t* build);

#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_VERSION_H_ */

