/************************************************
 *
 *  CROSSCONTROL AUX API
 *
 *  Battery.h
 *
 *  Version 2.15.2.0, 2020-07-08
 *
 *  (c) CrossControl 2010-2020
 *

 *  File description: 
 *  External battery status and settings (Only available on specific models)
 *
 *  Use the function GetBattery() to get a handle to pass on to the other functions. 
 *  Use the method Battery_release(BATTERYHANDLE) to return the handle.
 *
 *  Example Usage:
 *  @include battery_example.cpp
 *
 ************************************************/

#ifndef CC_BATTERY_H_
#define CC_BATTERY_H_

#include "CCAuxErrors.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  /*
   * Diagnostic timer data
   */
  typedef struct
  {
    uint32_t TotRunTimeMain;    /*! Total running time on main power (minutes)*/
    uint32_t TotRunTimeBattery; /*! Total running time on battery power (minutes)*/
    uint32_t RunTime_m20;       /*! Total runtime below -20 deg C (minutes)*/
    uint32_t RunTime_m20_0;     /*! Total runtime in range 0 to -20 deg C (minutes)*/
    uint32_t RunTime_0_40;      /*! Total runtime in range 0 to 40 deg C (minutes)*/
    uint32_t RunTime_40_60;     /*! Total runtime in range 40 to 60 deg C (minutes)*/
    uint32_t RunTime_60_70;     /*! Total runtime in range 60 to 70 deg C (minutes)*/
    uint32_t RunTime_70_80;     /*! Total runtime in range 70 to 80 deg C (minutes)*/
    uint32_t RunTime_Above80;   /*! Total runtime above 80 deg C (minutes)*/
  } BatteryTimerType;

  /*!
   * Current charging status of the battery.
   */
  typedef enum
  {
    ChargingStatus_NoCharge     = 0, /*!< The battery is not being charged. System is running on battery power. */
    ChargingStatus_Charging     = 1, /*!< The battery is currently being charged */
    ChargingStatus_FullyCharged = 2, /*!< The battery is fully chareged */
    ChargingStatus_TempLow      = 3, /*!< The temperature is too low to allow the battery to be charged */
    ChargingStatus_TempHigh     = 4, /*!< The temperature is too high to allow the battery to be charged */
    ChargingStatus_Unknown      = 5  /*!< There was an error determining the charging status */
  } ChargingStatus;

  /*!
   * Current power source of the computer.
   */
  typedef enum
  {
    PowerSource_Battery = 0,
    PowerSource_ExternalPower = 1
  } PowerSource;

  /*!
   * Error status.
   */
  typedef enum
  {
    ErrorStatus_NoError = 0,
    ErrorStatus_ThermistorTempSensor = 1,
    ErrorStatus_SecondaryTempSensor = 2,
    ErrorStatus_ChargeFail = 3,
    ErrorStatus_Overcurrent = 4,
    ErrorStatus_Init = 5
  } ErrorStatus;

  typedef void* BATTERYHANDLE;

  /*!
   * Factory function that creates instances of the Battery object.
   *
   * Supported Platform(s): XM
   *
   * @return BATTERYHANDLE to an allocated battery object. The returned
   * handle needs to be deallocated using the Battery_release(BATTERYHANDLE)
   * method when it's no longer needed.
   * Returns NULL if it fails to allocate memory.
   *
   * Example Usage:
   * @snippet battery_example.cpp Calling GetBattery
   */
  EXTERN_C
    CCAUXDLL_API
    BATTERYHANDLE
    CCAUXDLL_CALLING_CONV
    GetBattery(void);

  /*!
   * Delete the Battery object
   *
   * Supported Platform(s): XM.
   *
   * @return -
   *
   * Example Usage:
   * @snippet battery_example.cpp Calling Battery_release
   */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    Battery_release(BATTERYHANDLE);

  /*!
   * Is an external battery connected?
   *
   * Supported Platform(s): XM
   *
   * @param batteryIsPresent true if a battery is connected, otherwise false.
   *
   * @return -
   *
   * Example Usage:
   * @snippet battery_example.cpp Calling isBatteryPresent
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Battery_isBatteryPresent (BATTERYHANDLE, bool* batteryIsPresent);

  /*! 
   * Get battery voltage status.
   *
   * Supported Platform(s): XM
   *
   * @param batteryVoltagePercent the current voltage level of the battery, in percent [0..100].
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet battery_example.cpp Calling getBatteryVoltageStatus
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Battery_getBatteryVoltageStatus (BATTERYHANDLE, uint8_t* batteryVoltagePercent);

  /*! 
   * Get battery charging status.
   *
   * Supported Platform(s): XM
   *
   * @param status the current charging mode of the battery.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet battery_example.cpp Calling getBatteryChargingStatus
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Battery_getBatteryChargingStatus (BATTERYHANDLE, ChargingStatus* status);

  /*! 
   * Get the currently used power source.
   *
   * Supported Platform(s): XM
   *
   * @param status the current power source, external power or battery.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet battery_example.cpp Calling getPowerSource
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Battery_getPowerSource (BATTERYHANDLE, PowerSource* status);

  /*!
   * Get battery temperature.
   *
   * Supported Platform(s): XM
   *
   * @param temperature PCB Temperature in degrees Celsius.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet battery_example.cpp Calling getBatteryTemp
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Battery_getBatteryTemp (BATTERYHANDLE, int16_t* temperature);

  /*!
   * Get hardware error code. If hardware errors are found or other problems are discovered
   * by the battery pack, they are reported here. 
   *
   * Supported Platform(s): XM
   *
   * @param errorCode Error code. Zero means no error.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet battery_example.cpp Calling getHwErrorStatus
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Battery_getHwErrorStatus (BATTERYHANDLE, ErrorStatus* errorCode);

  /*!
   * Get battery diagnostic timer.
   *
   * Supported Platform(s): XM
   *
   * @param times Get a struct with the current diagnostic times.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet battery_example.cpp Calling getTimer
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Battery_getTimer (BATTERYHANDLE, BatteryTimerType* times);

  /*!
   * Get temperature interval of the battery.
   *
   * Supported Platform(s): XM
   *
   * @param minTemp Minimum measured temperature.
   * @param maxTemp Maximum measured temperature.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet battery_example.cpp Calling getMinMaxTemp
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Battery_getMinMaxTemp (BATTERYHANDLE, int16_t* minTemp, int16_t* maxTemp);

  /*!
   * Get battery hardware version (PCB revision).
   *
   * Supported Platform(s): XM
   *
   * @param buff Text output buffer.
   * @param len Maximum length of the output buffer.
   * If the actual length of the data is greater, an error will be returned.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet battery_example.cpp Calling getBatteryHWversion
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Battery_getBatteryHWversion (BATTERYHANDLE, char_t* buff, int32_t len);

  /*!
   * Get the battery software version
   *
   * Supported Platform(s): XM    
   *
   * @param major Major version number
   * @param minor Minor version  number
   * @param release Release version number
   * @param build Build version number
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet battery_example.cpp Calling getBatterySwVersion
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Battery_getBatterySwVersion (
				 BATTERYHANDLE, 
				 uint16_t* major,
				 uint16_t* minor,
				 uint16_t* release,
				 uint16_t* build);

  /*!
   * Get battery serial number.
   *
   * Supported Platform(s): XM    
   *
   * @param buff Text output buffer.
   * @param len Maximum length of the output buffer.
   * If the actual length of the data is greater, an error will be returned.
   * The serial number is 10 characters plus terminating zero, in total 11 bytes in size.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet battery_example.cpp Calling getBatterySerial
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Battery_getBatterySerial (BATTERYHANDLE, char_t* buff, int32_t len);

#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_BATTERY_H_ */


