/************************************************
 *
 *  CROSSCONTROL AUX API
 *
 *  Buzzer.h
 *
 *  Version 2.15.2.0, 2020-07-08
 *
 *  (c) CrossControl 2010-2020
 *

 *  File description: 
 *  Buzzer settings 
 *
 *  Use the function GetBuzzer() to get a handle to pass on to the other functions. 
 *  Use the method Buzzer_release(BUZZERHANDLE) to return the handle.
 *
 *  Example Usage:
 *  @include buzzer_example.cpp
 *
 ************************************************/
#ifndef CC_BUZZER_H 
#define CC_BUZZER_H

#include "CCAuxErrors.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* BUZZERHANDLE;

  /*!
   * Factory function that creates instances of the Buzzer object.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @return BUZZERHANDLE to an allocated Buzzer object. The returned
   * handle needs to be deallocated using the Buzzer_release(BUZZERHANDLE) 
   * method when it's no longer needed.
   * Returns NULL if it fails to allocate memory.
   *
   * Example Usage:
   * @snippet buzzer_example.cpp Calling GetBuzzer
   */
  EXTERN_C
    CCAUXDLL_API
    BUZZERHANDLE
    CCAUXDLL_CALLING_CONV
    GetBuzzer(void);

  /*!
   * Delete the Buzzer object.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @return -
   *
   * Example Usage:
   * @snippet buzzer_example.cpp Calling Buzzer_release
   */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV 
    Buzzer_release(BUZZERHANDLE);

  /*!
   * Get buzzer frequency.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param frequency Current frequency (700-10000 Hz).
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Buzzer_getFrequency(BUZZERHANDLE, uint16_t* frequency);

  /*!
   * Get buzzer volume.
   * Note: For platform compatibility, use Buzzer_getScaledVolume instead. This function may be removed in a future API version.
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param volume Current volume (0-51) (VS: 0-2000) (VI2: 0-4095).
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet buzzer_example.cpp Calling Buzzer_getVolume
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Buzzer_getVolume(BUZZERHANDLE, uint16_t* volume);

  /*!
   * Get scaled buzzer volume.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param volume Current volume 0-100%
   *
   * Due limitation in HW, readback value may differ slightly from the value that was set.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet buzzer_example.cpp Calling Buzzer_getScaledVolume
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Buzzer_getScaledVolume(BUZZERHANDLE, uint8_t* volume);

  /*!
   * Get buzzer trigger. The Buzzer is enabled when the trigger is enabled.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param trigger Current trigger status.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Buzzer_getTrigger(BUZZERHANDLE, bool* trigger);

  /*!
   * Set buzzer frequency.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param frequency Frequency to set (700-10000 Hz).
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet buzzer_example.cpp Calling Buzzer_setFrequency
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Buzzer_setFrequency(BUZZERHANDLE, uint16_t frequency);

  /*!
   * Set buzzer volume.
   * Note: For platform compatibility, use Buzzer_setScaledVolume instead. This function may be removed in a future API version.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param volume Volume to set (0-51) (VS: 0-2000) (VI2: 0-4095) (V700: 0-50)
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet buzzer_example.cpp Calling Buzzer_setVolume
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Buzzer_setVolume(BUZZERHANDLE, uint16_t volume);

  /*!
   * Set scaled buzzer volume.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param volume Volume to set 0-100%.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet buzzer_example.cpp Calling Buzzer_setScaledVolume
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Buzzer_setScaledVolume(BUZZERHANDLE, uint8_t volume);

  /*!
   * Set buzzer trigger. The Buzzer is enabled when the trigger is enabled.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param trigger Status to set.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Buzzer_setTrigger(BUZZERHANDLE, bool trigger);

  /*!
   * Buzzes for a specified time.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param time Time (ms) to buzz.
   * @param blocking Blocking or non-blocking function.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet buzzer_example.cpp Calling Buzzer_buzze
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Buzzer_buzze(BUZZERHANDLE, int32_t time, bool blocking);

#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_BUZZER_H */

