/************************************************
 *
 *  CROSSCONTROL AUX API
 *
 *  CCAuxTypes.h
 *
 *  Version 2.15.2.0, 2020-07-08
 *
 *  (c) CrossControl 2010-2020
 *

 *  File description: Common types
 *
 ************************************************/

#ifndef CCAUXTYPES_H_
#define CCAUXTYPES_H_

// CCAux API uses default integer types based on stdint.h with addition of float, double and character types. 
// stdint.h is not available on all platforms/systems (e.g. Visual Studio for Windows). 
// Define CCAUX_USE_STDINT_H to use the stdint.h from your build system instead of these defaults. 
// Define CCAUX_USER_TYPEDEFS to use your own header file entirely.
#ifdef CCAUX_USE_STDINT_H
#include <stdint.h>
typedef float             float32_t;
typedef double            float64_t;
typedef char              char_t;
#else
// You can also include your own typedef file.
#ifdef CCAUX_USER_TYPEDEFS
// This file is not part of CCAux, you must provide it yourself.
#include <ccaux_ext_typedefs.h>
#else

// CCAux default typdefs
typedef float             float32_t;
typedef double            float64_t;
typedef char              char_t;

#ifdef _MSC_VER
// Microsoft Visual Studio
#if (_MSC_VER < 1300) 
// Visual Studio 6 and Embedded Visual C++ 4 
typedef signed char       int8_t;
typedef signed short      int16_t;
typedef signed int        int32_t;
typedef unsigned char     uint8_t;
typedef unsigned short    uint16_t;
typedef unsigned int      uint32_t;
#else
// VS2003+
typedef signed __int8     int8_t;
typedef signed __int16    int16_t;
typedef signed __int32    int32_t;
typedef unsigned __int8   uint8_t;
typedef unsigned __int16  uint16_t;
typedef unsigned __int32  uint32_t;
#endif
typedef signed __int64    int64_t;
typedef unsigned __int64  uint64_t;

#else // _MSC_VER
// Other compilers (e.g gcc)
typedef signed char       int8_t;
typedef signed short      int16_t;
typedef signed int        int32_t;
typedef unsigned char     uint8_t;
typedef unsigned short    uint16_t;
typedef unsigned int      uint32_t;
#ifdef X64
typedef long int          int64_t;
#else
//typedef long long int     int64_t;
#endif
#ifdef X64
typedef unsigned long int uint64_t;
#else
//typedef unsigned long long int  uint64_t;
#endif

#endif // _MSC_VER
#endif // CCAUX_USER_TYPEDEFS
#endif // CCAUX_USE_STDINT


#ifdef __cplusplus
namespace CrossControl
{
#endif

#ifndef __cplusplus
#define bool int
#endif
  
/*!
 * Voltage type enumeration
 */
typedef enum 
{
  VOLTAGE_24VIN = 0, /*! < 24VIN            */
  VOLTAGE_24V,       /*! < 24V              */
  VOLTAGE_12V,       /*! < 12V              */
  VOLTAGE_12VID,     /*! < 12VID            */
  VOLTAGE_5V,        /*! < 5V               */
  VOLTAGE_3V3,       /*! < 3.3V             */
  VOLTAGE_VTFT,      /*! < VTFT             */
  VOLTAGE_5VSTB,     /*! < 5VSTB            */
  VOLTAGE_1V9,       /*! < 1.9V             */
  VOLTAGE_1V8,       /*! < 1.8V             */
  VOLTAGE_1V5,       /*! < 1.5V             */
  VOLTAGE_1V2,       /*! < 1.2V             */
  VOLTAGE_1V05,      /*! < 1.05V            */
  VOLTAGE_1V0,       /*! < 1.0V             */
  VOLTAGE_0V9,       /*! < 0.9V             */
  VOLTAGE_VREF_INT,  /*! < SS internal VRef */
  VOLTAGE_24V_BACKUP,/*! < 24V backup capacitor */
  VOLTAGE_2V5,       /*! < 2.5V             */
  VOLTAGE_1V1,       /*! < 1.1V             */
  VOLTAGE_1V3_PER,   /*! < 1.3V_PER         */
  VOLTAGE_1V3_VDDA,  /*! < 1.3V_VDDA        */
  VOLTAGE_3V3STBY,   /*! < 3.3V STBY VC     */
  VOLTAGE_VPMIC,     /*! < V PMIC VC        */
  VOLTAGE_VMAIN,     /*! < V MAIN VC        */
  VOLTAGE_UB,        /*! < V UB VI2         */
  VOLTAGE_AI_1,      /*! < V AI_1 VI2       */
  VOLTAGE_AI_2,      /*! < V AI_2 VI2       */
  VOLTAGE_AI_3,      /*! < V AI_3 VI2       */
  VOLTAGE_END        /*! < Marks end of voltages - list */
}VoltageEnum;

/*! Light sensor operation ranges.*/
typedef enum
{
  RangeStandard = 0, /*! Light sensor operation range standard */
  RangeExtended = 1  /*! Light sensor operation range extended */
} LightSensorOperationRange;

/*! Light sensor sampling modes.*/
typedef enum
{
  SamplingModeStandard = 0, /*! Standard sampling mode.*/
  SamplingModeExtended,     /*!Extended sampling mode.*/
  SamplingModeAuto         /*! Auto switch between standard and extended
                 sampling mode depending on saturation.*/
} LightSensorSamplingMode;

/*!
 * Enable/disable enumeration
 */
typedef enum
{
  Disabled = 0, /*! The setting is disabled or turned off */
  Enabled = 1   /*! The setting is enabled or turned on */
} CCStatus;

/*!
 * Error code enumeration
 */
typedef enum
{
  ERR_SUCCESS = 0,                        /*! Success                        */
  ERR_OPEN_FAILED = 1,                    /*! Open failed                    */
  ERR_NOT_SUPPORTED = 2,                  /*! Not supported                  */
  ERR_UNKNOWN_FEATURE = 3,                /*! Unknown feature                */
  ERR_DATATYPE_MISMATCH = 4,              /*! Datatype mismatch              */
  ERR_CODE_NOT_EXIST = 5,                 /*! Code doesn't exist             */
  ERR_BUFFER_SIZE = 6,                    /*! Buffer size error              */
  ERR_IOCTRL_FAILED = 7,                  /*! IoCtrl operation failed        */
  ERR_INVALID_DATA = 8,                   /*! Invalid data                   */
  ERR_INVALID_PARAMETER = 9,              /*! Invalid parameter              */
  ERR_CREATE_THREAD = 10,                 /*! Failed to create thread        */
  ERR_IN_PROGRESS = 11,                   /*! Operation in progress          */
  ERR_CHECKSUM = 12,                      /*! Checksum error                 */
  ERR_INIT_FAILED = 13,                   /*! Initalization failed           */
  ERR_VERIFY_FAILED = 14,                 /*! Failed to verify               */
  ERR_DEVICE_READ_DATA_FAILED = 15,       /*! Failed to read from device     */
  ERR_DEVICE_WRITE_DATA_FAILED = 16,      /*! Failed to write to device      */
  ERR_COMMAND_FAILED = 17,                /*! Command failed                 */
  ERR_EEPROM = 18,                        /*! Error in EEPROM memory         */
  ERR_JIDA_TEMP = 19,                     /*! Failed to get JIDA temperature */
  ERR_AVERAGE_CALC_STARTED = 20,          /*! Calculation already started    */
  ERR_NOT_RUNNING = 21,                   /*! Thread isn't running           */
  ERR_I2C_EXPANDER_READ_FAILED = 22,      /*! I2C read failure               */
  ERR_I2C_EXPANDER_WRITE_FAILED = 23,     /*! I2C write failure              */
  ERR_I2C_EXPANDER_INIT_FAILED = 24,      /*! I2C initialization failure     */
  ERR_NEWER_SS_VERSION_REQUIRED = 25,     /*! SS version too old             */
  ERR_NEWER_FPGA_VERSION_REQUIRED = 26,   /*! FPGA version too old           */
  ERR_NEWER_FRONT_VERSION_REQUIRED = 27,  /*! FRONT version too old          */
  ERR_TELEMATICS_GPRS_NOT_AVAILABLE = 28, /*! GPRS module not available      */
  ERR_TELEMATICS_WLAN_NOT_AVAILABLE = 29, /*! WLAN module not available      */
  ERR_TELEMATICS_BT_NOT_AVAILABLE = 30,   /*! Bluetooth module not available */
  ERR_TELEMATICS_GPS_NOT_AVAILABLE = 31,  /*! GPS module not available       */
  ERR_MEM_ALLOC_FAIL = 32,                /*! Failed to allocate memory      */
  ERR_JOIN_THREAD = 33,                   /*! Failed to join thread          */
  ERR_INVALID_STARTUP_TRIGGER = 34,       /*! Start-up trigger or button configuration set to an invalid value, see enum TriggerConf for more details. */
  ERR_END                                 /*! Marks end of errors - list */
} eErr;

/*!
 * The available deinterlace modes
 */
typedef enum
{
  DeInterlace_Even = 0, /*! Use only even rows from the interlaced input stream */
  DeInterlace_Odd = 1, /*! Use only odd rows from the interlaced input stream */
  DeInterlace_BOB = 2 /*! Use alternating odd and even rows from the interlaced input stream */
} DeInterlaceMode;

/*!
 * The available analog video channels
 */
typedef enum
{
  Analog_Channel_1 = 0, /* Analog video channel 1 */
  Analog_Channel_2 = 1, /* Analog video channel 2 */
  Analog_Channel_3 = 2, /* Analog video channel 3 */
  Analog_Channel_4 = 3, /* Analog video channel 4 */
  Analog_channel_END    /* Marks end of analog channel - list */
} VideoChannel;

/* Struct used for received video */
struct received_video
{
  uint16_t received_width;
  uint16_t received_height;
  uint8_t received_framerate;
};

/* Struct used for I2c-accesses to decoder */
struct video_dec_command
{
  uint8_t decoder_register;
  uint8_t register_value;
};

/* Version info struct */
struct version_info
{
  uint8_t major;   /*!< version major number */
  uint8_t minor;   /*!< version minor number */
  uint8_t release; /*!< version release number */
  uint8_t build;   /*!< version build number */
};

/* Video standard */
/* The video decoder auto detect the video standard */
typedef enum
{
  STD_M_J_NTSC = 0,         /*! (M,J) NTSC ITU-R BT.601 */
  STD_B_D_G_H_I_N_PAL = 1,  /*! (B, D, G, H, I, N) PAL ITU-R BT.601 */
  STD_M_PAL = 2,            /*! (M) PAL ITU-R BT.601 */
  STD_PAL = 3,              /*! PAL-Nc ITU-R BT.601 */
  STD_NTSC = 4,             /*! NTSC 4.43 ITU-R BT.601 */
  STD_SECAM = 5             /*! SECAM ITU-R BT.601 */
} videoStandard;

/* Available video rotation */
typedef enum
{
  RotNone = 0,
  Rot90,
  Rot180,
  Rot270
} VideoRotation;

typedef struct version_info VersionType;

/* Buzzer setup struct */
typedef struct
{
  uint16_t frequency; /*!< buzzer frequency*/
  uint16_t volume; /*!< buzzer volume*/
} BuzzerSetup;

/* Led timing setup struct */
typedef struct
{
  uint8_t onTime; /*!< LED on time in 10ms*/
  uint8_t offTime; /*!< LED off time in 10ms*/
  uint8_t idleTime; /*!< LED idle time in 100ms*/
  uint8_t nrOfPulses; /*!< Pulses per sequences*/
} LedTimingType;

/* For FPGA leds */
typedef struct
{
  uint8_t ledNbr; /*!< Number of LED*/
  uint8_t onTime; /*!< LED on time in 10ms*/
  uint8_t offTime; /*!< LED off time in 10ms*/
  uint8_t idleTime; /*!< LED idle time in 100ms*/
  uint8_t nrOfPulses; /*!< Pulses per sequences*/
} FpgaLedTimingType;

/* Led color struct */
typedef struct
{
  uint8_t red; /*!< Red color intensity 0-0x0F*/
  uint8_t green; /*!< Green color intensity 0-0x0F*/
  uint8_t blue; /*!< Blue color intensity 0-0x0F*/
} LedColorMixType;

/*!
 * Can frame type settings
 */
typedef enum
{
  FrameStandard,
  FrameExtended,
  FrameStandardExtended
} CanFrameType;

/*!
 * Trigger configuration enumeration. Valid settings for enabling of front button and external on/off signal.
 * For platforms XM, XL and XA platforms, front button and on/off (ignition) signal can be configured. 
 *
 * For the VC platform, CI state activity and Can data reception can also be used as wakeup sources from suspend mode.
 * bit 0 - enable wakeup by front button (from OFF and suspend mode)
 * bit 1 - enable wakeup by on/off (ignition) signal (from OFF and suspend mode)
 * bit 2 - enable wakeup by CAN activity (from suspend mode, VC only)
 * bit 3 - enable wakeup by CI (Configurabe input) state change (from suspend mode, VC only)
 *
 * Note that there must always be a way to start the unit from shutdown mode. Therefore, at least one of the following must be true: 
 * - Front button enabled as start-up trigger AND (CCpilot VC) at least one button configured as start-up trigger
 * - External on/off (ignition) signal configured as start-up trigger.
 */
typedef enum
{
  Front_Button_Enabled = 1,           /*!< Front button is enabled for startup and wake-up */
  OnOff_Signal_Enabled = 2,           /*!< The external on/off signal is enabled for startup and wake-up */
  Both_Button_And_Signal_Enabled = 3, /*!< Both of the above are enabled */
  CAN_Button_Activity = 5,            /*!< VC platform, wake up on CAN and Buttons */
  CAN_OnOff_Activity = 6,             /*!< VC platform, wake up on CAN and On/Off/Ignition signal */
  CAN_Button_OnOff_Activity = 7,      /*!< VC platform, wake up on CAN, Buttons and On/Off/Ignition signal */
  CI_Button_Activity = 9,             /*!< VC platform, wake up on CI and Button State Change */
  CI_OnOff_Activity = 10,             /*!< VC platform, wake up on CI and On/Off/Ignition signal State Change */
  CI_Button_OnOff_Activity = 11,      /*!< VC platform, wake up on CI, Button and On/Off/Ignition signal State Change */
  CI_CAN_Button_Activity = 13,        /*!< VC platform, wake up on CI, CAN and Button State Change */
  CI_CAN_OnOff_Activity = 14,         /*!< VC platform, wake up on CI, CAN and On/Off/Ignition signal State Change */
  All_Events = 15,                    /*!< VC platform, wake up on all events */
  Last_trigger_conf
} TriggerConf;

/*!
 * Button and on/off signal actions.
 */
typedef enum
{
  NoAction = 0, /*!< No action taken */
  ActionSuspend = 1, /*!< The system enters suspend mode */
  ActionShutDown = 2 /*!< The system shuts down */
} PowerAction;

/*!
 * Current status for front panel button and on/off signal. If any of them generate a suspend or shutdown event, it may also be read, briefly.
 * When the button/signal is released, typically BPTS_Suspend or BPTS_ShutDown follows.
 * Note: Do not rely on getting BPTS_ShutDown or BPTS_Suspend from user applications. The system is likely to start shutting down before the status can be read.
 * Instead, use the PowerMgr class for handling system shutdown/suspend events.
 */
typedef enum
{
  BPTS_No_Change    = 0,  /*!< No change */
  BPTS_ShutDown    = 1,  /*!< A shutdown has been initiated since the front panel button has been pressed longer than the set FrontBtnShutDownTrigTime */
  BPTS_Suspend    = 2,  /*!< Suspend mode has been initiated since the front panel button has been pressed (shortly) and suspend mode is enabled */
  BPTS_Restart    = 3,  /*!< Not currently in use */
  BPTS_BtnPressed    = 4,  /*!< The front panel button is currently pressed. It has not been released and it has not yet been held longer than FrontBtnShutDownTrigTime. */
  BPTS_BtnPressedLong = 5,  /*!< The front panel button is currently pressed. It has not been released and it has been held longer than FrontBtnShutDownTrigTime. */
  BPTS_SignalOff    = 6,  /*!< The external on/off signal is low, but not yet long enough for the ExtOnOffSigSuspTrigTime. */
  BPTS_END                /*!< Mark end of BPTS - list */
} ButtonPowerTransitionStatus;

/*!
 * Overcurrent detection status.
 */
typedef enum
{
  OCD_OK = 0,       /*!< Normal operation, no overcurrent condition detected */
  OCD_OC = 1,       /*!< Overcurrent has been detected, power has therefore been turned off, but may be functioning again if the problem disappeared after re-test*/
  OCD_POWER_OFF = 2 /*!< Overcurrent has been detected, power has been permanently turned off */
} OCDStatus;

/*!
 * Diagnostic timer data
 */
typedef struct
{
  uint32_t TotRunTime; /*! Total running time (minutes)*/
  uint32_t TotSuspTime; /*! Total suspend time (minutes)*/
  uint32_t TotHeatTime; /*! Total heating time (minutes)*/
  uint32_t RunTime40_60; /*! Total runtime in 40-60deg (minutes)*/
  uint32_t RunTime60_70; /*! Total runtime in 60-70deg (minutes)*/
  uint32_t RunTime70_80; /*! Total runtime in 70-80deg (minutes)*/
  uint32_t Above80RunTime;/*! Total runtime above 80deg (minutes)*/
} TimerType;

/*!
 * Jida temperature sensor types
 */
typedef enum
{
  TEMP_CPU = 0,
  TEMP_BOX = 1,
  TEMP_ENV = 2,
  TEMP_BOARD = 3,
  TEMP_BACKPLANE = 4,
  TEMP_CHIPSETS = 5,
  TEMP_VIDEO = 6,
  TEMP_OTHER = 7
} JidaSensorType;

/*!
 * Upgrade Action enumeration
 */
enum UpgradeAction
{
  UPGRADE_INIT,         /*! Initiating, checking for compatibility etc */
  UPGRADE_PREP_COM,       /*! Preparing communication */
  UPGRADE_READING_FILE,     /*! Opening and reading the supplied file */
  UPGRADE_CONVERTING_FILE,    /*! Converting the mcs format to binary format */
  UPGRADE_FLASHING,       /*! Flashing the file */
  UPGRADE_VERIFYING,        /*! Verifying the programmed image */
  UPGRADE_COMPLETE,       /*! Upgrade was finished */
  UPGRADE_COMPLETE_WITH_ERRORS  /*! Upgrade finished prematurely, see errorCode for the reason of failure */
};

/*!
 * Upgrade Status
 */
typedef struct
{
  enum UpgradeAction currentAction; /*! The current action. */
  uint8_t percent; /*! Represents the percentage of completion of the current action */
  eErr errorCode; /*! The status of the operation */
} UpgradeStatus;

/*!
 * Enumeration of standard colors
 */
typedef enum
{
  RED = 0,     /*! RGB 0xF, 0x0, 0x0*/
  GREEN,       /*! RGB 0x0, 0xF, 0x0*/
  BLUE,        /*! RGB 0x0, 0x0, 0xF*/
  CYAN,        /*! RGB 0x0, 0xF, 0xF*/
  MAGENTA,     /*! RGB 0xF, 0x0, 0xF*/
  YELLOW,      /*! RGB 0xF, 0xF, 0x0*/
  UNDEFINED_COLOR   /*! Returns if color is not a standard color*/
} CCAuxColor;

/*!
 * Enumeration of RS4XX ports (1-4)
 */
typedef enum
{
  RS4XXPort1 = 1,
  RS4XXPort2,
  RS4XXPort3,
  RS4XXPort4
} RS4XXPort;

/*!
 * Enumeration of ConfigurableInput modes
 */
typedef enum
{
    CFGIN_NOT_IN_USE = 0,   /*! Disable configurable input measurement. */
    CFGIN_HI_SWITCH,        /*! Read digital input value with CfgIn_getValue */
    CFGIN_LOW_SWITCH,       /*! Read digital input value with CfgIn_getValue */
    CFGIN_VOLTAGE_2V5,      /*! Read voltage input value with CfgIn_getValue, 2.5V range */
    CFGIN_VOLTAGE_5V,       /*! Read voltage input value with CfgIn_getValue, 5V range */
    CFGIN_RESISTANCE,       /*! Read resistance input value with CfgIn_getValue, ports 1-4 only */
    CFGIN_FREQ_FLOATING,    /*! Read frequency value with CfgIn_getPwmValue */
    CFGIN_FREQ_PULLUP,      /*! Read frequency value with CfgIn_getPwmValue */
    CFGIN_FREQ_PULLDOWN,    /*! Read frequency value with CfgIn_getPwmValue */
    CFGIN_RESISTANCE_500,   /*! Read resistance input value with CfgIn_getValue, 0-500Ohm range, VA only, ports 1-4 only */
    CFGIN_CURRENT_4_20,     /*! Read current input value with CfgIn_getValuerange 4-20 mA, VA only. */
    CFGIN_VOLTAGE_10V,      /*! Read voltage input value with CfgIn_getValue, 10V range, VA only*/
    CFGIN_VOLTAGE_32V,      /*! Read voltage input value with CfgIn_getValue, 32V range, VA only*/
    CFGIN_DIGITAL_PD_5V,    /*! Read digital input value with CfgIn_getValue, pull-down, 5V range, 2.5V threshold, VA only */
    CFGIN_DIGITAL_PD_10V,   /*! Read digital input value with CfgIn_getValue, pull-down, 10V range, 5V threshold, VA only*/
    CFGIN_DIGITAL_PD_32V,   /*! Read digital input value with CfgIn_getValue, pull-down, 32V range, 10V threshold, VA only*/
    CFGIN_DIGITAL_F_5V,     /*! Read digital input value with CfgIn_getValue, floating, 5V range, 2.5V threshold, VA only*/
    CFGIN_DIGITAL_F_10V,    /*! Read digital input value with CfgIn_getValue, floating, 10V range, 5V threshold, VA only*/
    CFGIN_DIGITAL_F_32V,    /*! Read digital input value with CfgIn_getValue, floating, 32V range, 10V threshold, VA only*/
    CFGIN_DIGITAL_PU_5V,    /*! Read digital input value with CfgIn_getValue, pull-up, 5V range, 2.5V threshold, VA only, ports 5-8 only */
    CFGIN_DIGITAL_PU_10V,   /*! Read digital input value with CfgIn_getValue, pull-up, 10V range, 5V threshold, VA only, ports 5-8 only */
    CFGIN_DIGITAL_PU_32V,   /*! Read digital input value with CfgIn_getValue, pull-up, 32V range, 10V threshold, VA only, ports 5-8 only */
    CFGIN_FREQ_PD_5V,       /*! Read frequency value with CfgIn_getFrequencyValue, pull-down, 5V range, 2.5V threshold, VA only */
    CFGIN_FREQ_PD_10V,      /*! Read frequency value with CfgIn_getFrequencyValue, pull-down, 10V range, 5V threshold, VA only */
    CFGIN_FREQ_PD_32V,      /*! Read frequency value with CfgIn_getFrequencyValue, pull-down, 32V range, 10V threshold, VA only */
    CFGIN_FREQ_F_5V,        /*! Read frequency value with CfgIn_getFrequencyValue, floating, 5V range, 2.5V threshold, VA only */
    CFGIN_FREQ_F_10V,       /*! Read frequency value with CfgIn_getFrequencyValue, floating, 10V range, 5V threshold, VA only */
    CFGIN_FREQ_F_32V,       /*! Read frequency value with CfgIn_getFrequencyValue, floating, 32V range, 10V threshold, VA only */
    CFGIN_FREQ_PU_5V,       /*! Read frequency value with CfgIn_getFrequencyValue, pull-up, 5V range, 2.5V threshold, VA only, ports 5-8 only */
    CFGIN_FREQ_PU_10V,      /*! Read frequency value with CfgIn_getFrequencyValue, pull-up, 10V range, 5V threshold, VA only, ports 5-8 only */
    CFGIN_FREQ_PU_32V,      /*! Read frequency value with CfgIn_getFrequencyValue, pull-up, 32V range, 10V threshold, VA only, ports 5-8 only */
    CFGIN_VS_FreqInMode1,   /*! Mode 1, High level 3420mV, Low level 2540mV, VS only */
    CFGIN_VS_FreqInMode1PU, /*! Mode 1, High level 3420mV, Low level 2540mV, with pull-up, VS only */
    CFGIN_VS_FreqInMode2,   /*! Mode 2, High level 6280mV, Low level 1520mV, VS only */
    CFGIN_VS_FreqInMode2PU,  /*! Mode 2, High level 6280mV, Low level 1520mV, with pull-up, VS only */
    CFGIN_MAX
} CfgInModeEnum;

/*!
 * Enum CfgInModeEnum members were renamed. This is for backward compatibility with previous versions.
 */
#define CFGIN_VOLTAGE_3V3 CFGIN_VOLTAGE_2V5
#define CFGIN_VOLTAGE_5VPD CFGIN_VOLTAGE_5V

/*!
 * Enumeration of Button Configuration (bitfield representation)
 */
typedef enum
{
    BUTTON_ONLY_MP_ACTION = 0x00,                  /*! Buttons are only read by Main Processor, i.e. normal button action which is handled in application space */
    BUTTON_AS_STARTUP_TRIG = 0x02,                 /*! Set button to be used as startup trigger, in addition to MP application event */
    BUTTON_AS_ACTION_TRIG = 0x04,                  /*! Set button to trigger suspend, shutdown or hard shutdown actions */
    BUTTON_AS_ACTION_STARTUP_TRIG = 0x06,          /*! Set button to trigger startup, suspend, shutdown or hard shutdown actions */
    BUTTON_AS_BACKLIGHT_DECREASE = 0x08,           /*! Set button to act as backlight decrease button */
    BUTTON_AS_BACKLIGHT_DECR_STARTUP_TRIG = 0x0A,  /*! Set button to act as backlight decreaste and startup trigger */
    BUTTON_AS_BACKLIGHT_INCREASE = 0x0C,           /*! Set button to act as backlight increase button */
    BUTTON_AS_BACKLIGHT_INCR_STARTUP_TRIG = 0x0E   /*! Set button to act as backlight increase button and startup trigger */
} ButtonConfigEnum;

/*!
 * Enumeration of Bootmodes
 */
typedef enum
{
  BOOTMODE_EMMC = 0,      /*! Boot from eMMC */
  BOOTMODE_SD,            /*! Boot from SD   */
  BOOTMODE_SERIAL,        /*! Serial download mode */
  BOOTMODE_RESCUE,        /*! Boot to rescue system */
  BOOTMODE_RESCUE_SPECIAL /*! Boot to rescue system. Plaform depended special. */
} BootModeEnum;

/*!
 * Enumeration of OnOff/Ignition/KeySwitch signal trigger modes
 */
typedef enum
{
  CONFIG_ONOFF_EDGE_TRIGGER = 0,	/*! OnOff/Ignition/KeySwitch signal trigger on signal edge (default) */
  CONFIG_ONOFF_LEVEL_TRIGGER 	    /*! OnOff/Ignition/KeySwitch signal trigger on signal level */
} ConfigOnOffTriggerMode;


typedef enum
{
  PowerOutput1 = 0,
  PowerOutput2,
  PowerOutput3,
  PowerOutput4,
  PowerOutput5,  
  PowerOutput6,
  PowerOutputMax
} PowerOutput;

/*!
 * Enumeration of system running modes
 */
typedef enum
{
  SYSTEMMODE_Startup = 0,
  SYSTEMMODE_StartupRescue = 1,
  SYSTEMMODE_StartupRescueFactoryReset = 2,
  SYSTEMMODE_NormalRunning = 3,
  SYSTEMMODE_RescueRunning = 4, 
  SYSTEMMODE_RescueRunningFactoryReset = 5,
  SYSTEMMODE_Unknown = 6
} SystemMode;


#define MAX_BEEP_VOLUME 2000

#ifdef __cplusplus
}
#endif

#endif /*CCAUXTYPES_H_*/

