/************************************************
 *
 *  CROSSCONTROL AUX API
 *
 *  CanSetting.h
 *
 *  Version 2.15.2.0, 2020-07-08
 *
 *  (c) CrossControl 2010-2020
 *

 *  File description: 
 *  Can settings 
 *
 *  Use the function GetCanSetting() to get a handle to pass on to the other functions. 
 *  Use the method CanSetting_release(CANSETTINGHANDLE) to return the handle.
 *
 *  Example Usage:
 *  @include cansetting_example.cpp
 *
 ************************************************/
#ifndef CC_CAN_SETTING_H 
#define CC_CAN_SETTING_H

#include "CCAuxErrors.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* CANSETTINGHANDLE;

  /*!
   * Factory function that creates instances of the CanSetting object.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @return CANSETTINGHANDLE to an allocated CanSetting object. The returned
   * handle needs to be deallocated using the CanSetting_release(CANSETTINGHANDLE) 
   * method when it's no longer needed.
   * Returns NULL if it fails to allocate memory.
   *
   * Example Usage:
   * @snippet cansetting_example.cpp Calling GetCanSetting
   */
  EXTERN_C
    CCAUXDLL_API
    CANSETTINGHANDLE
    CCAUXDLL_CALLING_CONV
    GetCanSetting(void);
  
  /*!
   * Delete the CanSetting object.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @return -
   *
   * Example Usage:
   * @snippet cansetting_example.cpp Calling CanSetting_release
   */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    CanSetting_release(CANSETTINGHANDLE);

  /*!
   * Get Baud rate
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param net CAN net (1-4) to get settings for.
   * @param baudrate CAN baud rate (kbit/s).
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet cansetting_example.cpp Calling CanSetting_getBaudrate
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    CanSetting_getBaudrate (CANSETTINGHANDLE, uint8_t net, uint16_t* baudrate);

  /*!
   * Get frame type
   *
   * Supported Platform(s): XL, XM
   *
   * @param net CAN net (1-4) to get settings for.
   * @param frameType CAN frame type
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet cansetting_example.cpp Calling CanSetting_getFrameType
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    CanSetting_getFrameType (CANSETTINGHANDLE, uint8_t net, CanFrameType* frameType);

  /*!
   * Set Baud rate. The changes will take effect after a restart.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param net CAN net (1-4).
   * @param baudrate CAN baud rate (kbit/s). The driver will calculate the best
   * supported baud rate if it does not support the given baud rate.
   * The maximum baud rate is 1000 kbit/s.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    CanSetting_setBaudrate (CANSETTINGHANDLE, uint8_t net, uint16_t baudrate);

  /*!
   * Set frame type. The changes will take effect after a restart.
   *
   * Supported Platform(s): XL, XM
   *
   * @param net CAN net (1-4).
   * @param frameType CAN frameType
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    CanSetting_setFrameType (CANSETTINGHANDLE, uint8_t net, CanFrameType frameType);

#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_CAN_SETTING_H */

