/************************************************
 *
 *  CROSSCONTROL AUX API
 *
 *  Config.h
 *
 *  Version 2.15.2.0, 2020-07-08
 *
 *  (c) CrossControl 2010-2020
 *

 *  File description: 
 *  Configuration of various settings
 *
 *  Use the function GetConfig() to get a handle to pass on to the other functions. 
 *  Use the method Config_release(CONFIGHANDLE) to return the handle.
 *
 *  Example Usage:
 *  @include config_example.cpp
 *
 ************************************************/
#ifndef CC_CONFIG_H 
#define CC_CONFIG_H

#include "CCAuxErrors.h"
#include "CCAuxTypes.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* CONFIGHANDLE;

  /*!
   * Bit defines for getVideoStartupPowerConfig and setVideoStartupPowerConfig
   */
  const uint8_t Video1Conf = (1 << 0); /*! Video channel 1 config */
  const uint8_t Video2Conf = (1 << 1); /*! Video channel 2 config */
  const uint8_t Video3Conf = (1 << 2); /*! Video channel 3 config */
  const uint8_t Video4Conf = (1 << 3); /*! Video channel 4 config */

  /*!
   * Factory function that creates instances of the Config object.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @return CONFIGHANDLE to an allocated Config object. The returned
   * handle needs to be deallocated using the Config_release(CONFIGHANDLE) 
   * method when it's no longer needed.
   * Returns NULL if it fails to allocate memory.
   *
   * Example Usage:
   * @snippet config_example.cpp Calling GetConfig
   */
  EXTERN_C
    CCAUXDLL_API 
    CONFIGHANDLE
    CCAUXDLL_CALLING_CONV
    GetConfig();

  /*!
   * Delete the Config object.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @return -
   *
   * Example Usage:
   * @snippet config_example.cpp Calling Config_release    
   */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    Config_release(CONFIGHANDLE);

  /*!
   * Get Start-up trigger configuration. Is the front button and/or the external on/off (ignition) signal
   * enabled as triggers for startup and wake up from suspended mode?
   * VC platform: CI state change and Can activity also available as wakeup triggers from suspend mode.
   * See enum TriggerConf for more details. 
   *
   * Supported Platform(s): XL, XM, XL5, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param config See enum TriggerConf.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet config_example.cpp Calling Config_getStartupTriggerConfig
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getStartupTriggerConfig(CONFIGHANDLE, TriggerConf* config);

  /*!
   * Get short button press action. Gets the configured action for a short button press:
   * NoAction, ActionSuspend or ActionShutDown.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2
   *
   * @param action The configured action.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getShortButtonPressAction(CONFIGHANDLE, PowerAction* action);

  /*!
   * Get long button press action. Gets the configured action for a long button press:
   * NoAction, ActionSuspend or ActionShutDown. A long button press is determined by the FrontBtnTrigTime.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2
   *
   * @param action The configured action.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getLongButtonPressAction(CONFIGHANDLE, PowerAction* action);

  /*!
   * Get On/Off signal action. Gets the configured action for an On/Off signal event:
   * NoAction, ActionSuspend or ActionShutDown. An On/Off signal event is determined by the ExtOnOffSigTrigTime.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param action The configured action.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getOnOffSigAction(CONFIGHANDLE, PowerAction* action);

  /*!
   * Get front button trigger time for long press.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2
   *
   * @param triggertime Time in milliseconds that the button has to
   * be pressed for the press to count as a long button press.
   * A button press twice this time will generate a hard shut down. If this time is set under 4000ms,
   * the hard shut down minimum time of 8s is used instead.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getFrontBtnTrigTime(CONFIGHANDLE, uint16_t* triggertime);

  /*!
   * Get external on/off signal trigger time.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param triggertime Time in seconds that the external signal
   * has to be low for the unit to enter suspend mode or shut down (trigger an action).
   * This time can be set from one second up to several years, if needed.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getExtOnOffSigTrigTime(CONFIGHANDLE, uint32_t* triggertime);

  /*!
   * Get Button Function Configuration
   *
   * Supported Platform(s): VC, VA, VS, VI2
   *
   * @param button_number Which button to configure (1-MAX_BUTTONS)
   * @param button_config Bitfield for button configuration, see enum ButtonConfigEnum for details.
   * 
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet buttons_example.cpp Calling Config_getButtonFunction
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getButtonFunction(CONFIGHANDLE, uint8_t button_number, ButtonConfigEnum* button_config);

  /*!
   * Get suspend mode maximum time.
   *
   * Supported Platform(s): XL, XM, XM9 (CC Linux), VC, VA, VS, VI2, V700
   *
   * @param maxTime Maximum suspend time in minutes. After this time in
   * suspended mode, the unit will shut down to save power. A value of 0
   * means that the automatic shut down function is not used.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getSuspendMaxTime(CONFIGHANDLE, uint16_t* maxTime);

  /*!
   * Get Can power at startup configuration. The status of Can power
   * at startup and at resume from suspended mode. At resume
   * from suspend, this setting overrides the setting of the
   * setCanPowerStatus function.
   *
   * Supported Platform(s): XL, XM, XS, XA
   *
   * @param status Enabled/Disabled
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getCanStartupPowerConfig(CONFIGHANDLE, CCStatus* status);

  /*!
   * Get Video power at startup configuration. The status of Video power
   * at startup and at resume from suspended mode. At resume
   * from suspend, this setting overrides the setting of the
   * setVideoPowerStatus function.
   *
   * Supported Platform(s): XL, XM, XS, XA
   *
   * @param config Bitwise representation of the four video channels. See the VideoXConf defines.
   * if the bit is 1, the power is enabled, else disabled.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getVideoStartupPowerConfig(CONFIGHANDLE, uint8_t* config);

  /*!
   * Get External fan power at startup configuration. The status
   * at startup and at resume from suspended mode. At resume
   * from suspend, this setting overrides the setting of the
   * setExtFanPowerStatus function.
   *
   * Supported Platform(s): XL, XM
   *
   * @param status Enabled/Disabled
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getExtFanStartupPowerConfig(CONFIGHANDLE, CCStatus* status);

  /*!
   * Get the voltage threshold required for startup. The external voltage
   * must be stable above this value for the unit to start up.
   * The default and minimum value is 9V. It could be set to a higher
   * value for a 24V system.
   *
   * Supported Platform(s): XL, XM
   *
   * @param voltage The current voltage setting. (9V .. 28V)
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getStartupVoltageConfig(CONFIGHANDLE, float64_t* voltage);

  /*!
   * Get the temperature limit for heating. When temperature is below this limit,
   * the system is internally heated until the temperature rises above the limit.
   * The default and minimum value is -25 degrees Celsius. The maximum value is +5 degrees Celsius.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA
   *
   * @param temperature The current heating limit, in degrees Celsius (-25 to +5)
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getHeatingTempLimit(CONFIGHANDLE, int16_t* temperature);

  /*!
   * Get power on start-up behavior. If enabled, the unit always starts
   * when power is turned on, disregarding the setting for StartupTriggerConfig at that time.
   * The StartupTriggerConfig still applies if the unit is shut down or suspended, without
   * removing the power supply.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param status Enabled/Disabled
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getPowerOnStartup(CONFIGHANDLE, CCStatus* status); 

  /*!
   * Set Start-up trigger configuration. Should the front button and/or the external on/off (ignition) signal
   * be enabled as triggers for startup and wake up from suspended mode?
   * VC,VA platforms: CI state change and Can activity also available as wakeup triggers from suspend mode.
   * See enum TriggerConf for more details. 
   * 
   * Supported Platform(s): XL, XM, XS, XA, VC, VA, VS, VI2
   *
   * @param conf See enum TriggerConf.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */	
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setStartupTriggerConfig(CONFIGHANDLE, TriggerConf conf);

  /*!
   * Set short button press action. Sets the configured action for a short button press:
   * NoAction, ActionSuspend or ActionShutDown.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2
   *
   * @param action The action to set.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet config_example.cpp Calling Config_setShortButtonPressAction
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setShortButtonPressAction(CONFIGHANDLE, PowerAction action);

  /*!
   * Set long button press action. Sets the configured action for a long button press:
   * NoAction, ActionSuspend or ActionShutDown. A long button press is determined by the FrontBtnTrigTime.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2
   *
   * @param action The action to set.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setLongButtonPressAction(CONFIGHANDLE, PowerAction action);

  /*!
   * Set On/Off signal action. Sets the configured action for an On/Off signal event:
   * NoAction, ActionSuspend or ActionShutDown. An On/Off signal event is determined by the ExtOnOffSigTrigTime.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param action The action to set.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setOnOffSigAction(CONFIGHANDLE, PowerAction action);

  /*!
   * Set front button trigger time for long press.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2
   *
   * @param triggertime Time in milliseconds that the button has to
   * be pressed for the press to count as a long button press.
   * A button press twice this time will generate a hard shut down. If this time is set under 4000ms,
   * the hard shut down minimum time of 8s is used instead.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setFrontBtnTrigTime(CONFIGHANDLE, uint16_t triggertime);

  /*!
   * Set external on/off signal trigger time.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param triggertime Time in seconds that the external signal
   * has to be low for the unit to enter suspend mode or shut down (trigger an action).
   * This time can be set from one second up to several years, if needed.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setExtOnOffSigTrigTime(CONFIGHANDLE, uint32_t triggertime);

  /*!
   * Set button function configuration
   *
   * Supported Platform(s): VC, VA, VS, VI2
   *
   * @param button_number Which button to configure (1-MAX_BUTTONS)
   * @param button_config Bitfield for button configuration, see enum ButtonConfigEnum for details.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setButtonFunction(CONFIGHANDLE, uint8_t button_number, ButtonConfigEnum button_config);

  /*!
   * Set suspend mode maximum time.
   *
   * Supported Platform(s): XL, XM, XM9 (CC Linux), VC, VA, VS, VI2, V700
   *
   * @param maxTime Maximum suspend time in minutes. After this time in
   * suspended mode, the unit will shut down to save power. A value of 0
   * means that this function is not used.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setSuspendMaxTime(CONFIGHANDLE, uint16_t maxTime);

  /*!
   * Set Can power at startup configuration. The status of Can power
   * at startup and at resume from suspended mode. At resume
   * from suspend, this setting overrides the setting of the
   * setCanPowerStatus function.
   *
   * Supported Platform(s): XL, XM, XS, XA
   *
   * @param status Enabled/Disabled
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setCanStartupPowerConfig(CONFIGHANDLE, CCStatus status);

  /*!
   * Set Video power at startup configuration. The status of Video power
   * at startup and at resume from suspended mode. At resume
   * from suspend, this setting overrides the setting of the
   * setVideoPowerStatus function.
   *
   * Supported Platform(s): XL, XM, XS, XA
   *
   * @param config Bitwise representation of the four video channels. See the VideoXConf defines.
   * if the bit is 1, the power is enabled, else disabled.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setVideoStartupPowerConfig(CONFIGHANDLE, uint8_t config);

  /*!
   * Set External fan power at startup configuration. The status
   * at startup and at resume from suspended mode. At resume
   * from suspend, this setting overrides the setting of the
   * setExtFanPowerStatus function.
   *
   * Supported Platform(s): XL, XM
   *
   * @param status Enabled/Disabled
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setExtFanStartupPowerConfig(CONFIGHANDLE, CCStatus status);

  /*!
   * Set the voltage threshold required for startup. The external voltage
   * must be stable above this value for the unit to start up.
   * The default and minimum value is 9V. It could be set to a higher
   * value for a 24V system.
   *
   * Supported Platform(s): XL, XM
   *
   * @param voltage The voltage to set (9V .. 28V).
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setStartupVoltageConfig(CONFIGHANDLE, float64_t voltage);

  /*!
   * Set the temperature limit for heating. When temperature is below this limit,
   * the system is internally heated until the temperature rises above the limit.
   * The default and minimum value is -25 degrees Celsius. The maximum value is +5 degrees Celsius.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA
   *
   * @param temperature The heating limit, in degrees Celsius (-25 to +5)
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setHeatingTempLimit(CONFIGHANDLE, int16_t temperature);

  /*!
   * Set power on start-up behavior. If enabled, the unit always starts
   * when power is turned on, disregarding the setting for StartupTriggerConfig at that time.
   * The StartupTriggerConfig still applies if the unit is shut down or suspended, without
   * removing the power supply.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param status Enabled/Disabled
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setPowerOnStartup(CONFIGHANDLE, CCStatus status);

  /*!
   * Set RS485 mode enabled or disabled for RS4XX port.
   *
   * Supported Platform(s): XA, XS
   *
   * @param port RS4XX port (RS4XXPort1-4)
   * @param enabled RS485 enabled (true/false)
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setRS485Enabled (CONFIGHANDLE, RS4XXPort port, bool enabled);

  /*!
   * Get RS485 mode configuration for RS4XX port.
   *
   * Supported Platform(s): XA, XS
   *
   * @param port RS4XX port (RS4XXPort1-4)
   * @param enabled Is the RS485 port enabled (true/false)
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getRS485Enabled (CONFIGHANDLE, RS4XXPort port, bool* enabled);
    
  /*!
   * Set OnOff/Ignition/KeySwitch signal trigger mode.
   *
   * Supported Platform(s): XM9, XL5, XA, XS, VC, VA, VS, VI2, V700
   *
   * @param mode Signal trigger mode. See ConfigOnOffTriggerMode for details
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setOnOffTriggerMode (CONFIGHANDLE, ConfigOnOffTriggerMode mode); 
    
  /*!
   * Get OnOff/Ignition/KeySwitch signal trigger mode.
   *
   * Supported Platform(s): XM9, XL5, XA, XS, VC, VA, VS, VI2, V700
   *
   * @param mode Signal trigger mode. See ConfigOnOffTriggerMode for details
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getOnOffTriggerMode (CONFIGHANDLE, ConfigOnOffTriggerMode* mode); 

  /*!
   * Set BootMode for next boot. Default mode is restored after boot.
   *
   * Supported Platform(s): VS, VI2, V700
   *
   * @param mode BootMode for next boot.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setNextBootMode (CONFIGHANDLE, BootModeEnum mode);

  /*!
   * Get BootMode for next boot
   *
   * Supported Platform(s): VS, VI2, V700
   *
   * @param mode Next BootMode
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getNextBootMode (CONFIGHANDLE, BootModeEnum *mode);

  /*!
   * Set OS Alive Monitoring
   *
   * Supported Platform(s): VS, VI2, V700
   *
   * @param enabled Enable/Disable OS Alive Monitoring
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setOSAliveMonitoring (CONFIGHANDLE, CCStatus enabled);

  /*!
   * Get OS Alive Monitoring
   *
   * Supported Platform(s): VS, VI2, V700
   *
   * @param enabled Is OS Alive monitoring enabled/disabled
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getOSAliveMonitoring (CONFIGHANDLE, CCStatus *enabled);

  /*!
   * Get OnOff signal state
   *
   * Supported Platform(s): XM9, XL5, XA, XS, VC, VA, VS, VI2, V700
   *
   * @param enabled Is OnOff signal enabled/disabled
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getOnOffSignalState (CONFIGHANDLE, CCStatus *enabled);

  /*!
  * Get power output startup configuration. Power outputs 1 and 2 can be set
  * to be turned on automatically at system startup.
  *
  * Supported Platform(s): VS  
  *
  * @param output Select power output (PowerOutput1 or PowerOutput2) .
  * @param enabled Is the power output configured to be enabled at startup?
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getDigPowerOutputStartupConfig(CONFIGHANDLE, PowerOutput output, CCStatus* enabled);

  /*!
  * Set power output startup configuration. Power outputs 1 and 2 can be set
  * to be turned on automatically at system startup.
  *
  * Supported Platform(s): VS  
  *
  * @param output Select power output (PowerOutput1 or PowerOutput2). 
  * @param enabled Configure the power output to be enabled at startup.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setDigPowerOutputStartupConfig(CONFIGHANDLE, PowerOutput output, CCStatus enabled);
  
  /*!
  * Get Short Beep Settings
  * The computer issues a short beep at certain system events such as boot or button press. 
  * This beep may be configured or disabled by setting any of the parameters to 0.
  * 
  * Supported Platform(s): VS, V700
  *
  * @param duration Short beep duration (ms)
  * @param frequency Short beep frequency (Hz)
  * @param volume Short beep volume (0-2000)
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getShortBeepSettings (CONFIGHANDLE, uint16_t* duration, uint16_t* frequency, uint16_t* volume);

  /*!
  * Set Short Beep Settings
  * The computer issues a short beep at certain system events such as boot or button press. 
  * This beep may be configured or disabled by setting any of the parameters to 0.
  *  
  * Supported Platform(s): VS, V700
  *
  * @param duration Short beep duration (ms)
  * @param frequency Short beep frequency (Hz)
  * @param volume Short beep volume (0-2000)
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setShortBeepSettings (CONFIGHANDLE, uint16_t duration, uint16_t frequency, uint16_t volume);

  /*!
  * Get Long Beep Settings
  * The computer issues a long beep at certain system events, for example a long button press. 
  * This beep may be configured or disabled by setting any of the parameters to 0.
  * 
  * Supported Platform(s): VS, V700
  *
  * @param duration Long beep duration (ms)
  * @param frequency Long beep frequency (Hz)
  * @param volume Long beep volume (0-2000)
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getLongBeepSettings (CONFIGHANDLE, uint16_t* duration, uint16_t* frequency, uint16_t* volume);

  /*!
  * Set Long Beep Settings
  * The computer issues a long beep at certain system events, for example a long button press. 
  * This beep may be configured or disabled by setting any of the parameters to 0.
  * 
  * Supported Platform(s): VS, V700
  *
  * @param duration Long beep duration (ms)
  * @param frequency Long beep frequency (Hz)
  * @param volume Long beep volume (0-2000)
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setLongBeepSettings (CONFIGHANDLE, uint16_t duration, uint16_t frequency, uint16_t volume);


#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_CONFIG_H */

