/************************************************
 *
 *  CROSSCONTROL AUX API
 *
 *  Diagnostic.h
 *
 *  Version 2.15.2.0, 2020-07-08
 *
 *  (c) CrossControl 2010-2020
 *

 *  File description: 
 *  Access to unit diagnostic data
 *
 *  Use the function GetDiagnostic() to get a handle to pass on to the other functions. 
 *  Use the method Diagnostic_release(DIAGNOSTICHANDLE) to return the handle.
 *
 *  Example Usage:
 *  @include diagnostic_example.cpp
 *
 ************************************************/
#ifndef CC_DIAGNOSTIC_H 
#define CC_DIAGNOSTIC_H

#include "CCAuxErrors.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* DIAGNOSTICHANDLE;

  /*!
  * Factory function that creates instances of the Diagnostic object.
  *
  * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
  *
  * @return DIAGNOSTICHANDLE to an allocated Diagnostic object. The returned
  * handle needs to be deallocated using the Diagnostic_release(DIAGNOSTICHANDLE)
  * method when it's no longer needed.
  * Returns NULL if it fails to allocate memory.
  *
  * Example Usage:
  * @snippet diagnostic_example.cpp Calling GetDiagnostic
  */
  EXTERN_C
    CCAUXDLL_API
    DIAGNOSTICHANDLE
    CCAUXDLL_CALLING_CONV
    GetDiagnostic(void);

  /*!
  * Delete the Diagnostic object.
  *
  * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
  *
  * @return -
  *
  * Example Usage:
  * @snippet diagnostic_example.cpp Calling Diagnostic_release    
  */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    Diagnostic_release(DIAGNOSTICHANDLE);

  /*!
  * Get System Supervisor temperature.
  *
  * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
  *
  * @param temperature System Supervisor temperature in degrees Celsius.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet diagnostic_example.cpp Calling Diagnostic_getSSTemp
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Diagnostic_getSSTemp (DIAGNOSTICHANDLE, int16_t* temperature);

  /*!
  * Get PCB temperature.
  *
  * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
  *
  * @param temperature PCB Temperature in degrees Celsius.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Diagnostic_getPCBTemp (DIAGNOSTICHANDLE, int16_t* temperature);

  /*!
  * Get Processor Module temperature.
  * This temperature is read from the Kontron JIDA API or Congatec CGOS API. These API's also has a number of other functions,
  * please see the JIDA/CGOS documentation for how to use them separately.
  * @param index Zero-based index of the temperature sensor. Different boards may have different number of sensors.
  * CCpilot XM and XL currently has 2 sensors, board and cpu. An error is returned if the index is not supported.
  * CCpliot XM 2.0 supports only one sensor, CPU temperature.
  *
  * Supported Platform(s): XL, XM
  *
  * @param temperature Temperature in degrees Celsius.
  * @param jst The type of sensor that is being read.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Diagnostic_getPMTemp (DIAGNOSTICHANDLE, uint8_t index, int16_t* temperature, JidaSensorType* jst);

  /*!
  * Get startup reason.
  *
  * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
  *
  * @param reason See DiagnosticCodes.h for startup codes.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Diagnostic_getStartupReason(DIAGNOSTICHANDLE, uint16_t* reason);

  /*!
  * Get shutdown reason.
  *
  * Supported Platform(s): XL, XM
  *
  * @param reason See DiagnosticCodes.h for shutdown codes.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Diagnostic_getShutDownReason(DIAGNOSTICHANDLE, uint16_t* reason);

  /*!
  * Get hardware error code. If hardware errors are found or other problems are discovered
  * by the SS, they are reported here. See DiagnosticCodes.h for error codes.
  *
  * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
  *
  * @param errorCode Error code. Zero means no error.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Diagnostic_getHwErrorStatus(DIAGNOSTICHANDLE, uint16_t* errorCode);

  /*!
  * Get diagnostic timer.
  *
  * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
  *
  * @param times Get a struct with the current diagnostic times.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet diagnostic_example.cpp Calling Diagnostic_getTimer
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Diagnostic_getTimer(DIAGNOSTICHANDLE, TimerType* times);

  /*!
  * Get diagnostic temperature interval of the unit.
  *
  * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
  *
  * @param minTemp Minimum measured PCB temperature.
  * @param maxTemp Maximum measured PCB temperature.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet diagnostic_example.cpp Calling Diagnostic_getMinMaxTemp
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Diagnostic_getMinMaxTemp(DIAGNOSTICHANDLE, int16_t* minTemp, int16_t* maxTemp);

  /*!
  * Get number of power cycles.
  *
  * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
  *
  * @param powerCycles Total number of power cycles.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Diagnostic_getPowerCycles(DIAGNOSTICHANDLE, uint16_t* powerCycles);

  /*!
  * Clear the HW error status (this function is used by the CrossControl service/daemon to log any hardware errors)
  *
  * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Diagnostic_clearHwErrorStatus(DIAGNOSTICHANDLE);

#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_DIAGNOSTIC_H */

