/************************************************
 *
 *  CROSSCONTROL AUX API
 *
 *  DigIO.h
 *
 *  Version 2.15.2.0, 2020-07-08
 *
 *  (c) CrossControl 2010-2020
 *

 *  File description: 
 *  Read digital inputs
 *
 *  Use the function GetDigIO() to get a handle to pass on to the other functions. 
 *  Use the method DigIO_release(DIGIOHANDLE) to return the handle.
 *
 *  Example Usage:
 *  @include digio_example.cpp
 *
 ************************************************/
#ifndef CC_DIGIO_H 
#define CC_DIGIO_H

#include "CCAuxErrors.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* DIGIOHANDLE;

  /*!
  * Bit defines for getDigIO
  */
  const uint8_t DigitalIn_1 = (1 << 0);
  const uint8_t DigitalIn_2 = (1 << 1);
  const uint8_t DigitalIn_3 = (1 << 2);
  const uint8_t DigitalIn_4 = (1 << 3);

  /*!
   * Factory function that creates instances of the DigIO object.  
   *
   * Supported Platform(s): XL, XM, XS, XA
   *
   * @return DIGIOHANDLE to an allocated DigIO object. The returned
   * handle needs to be deallocated using the DigIO_release(DIGIOHANDLE) 
   * method when it's no longer needed.
   * Returns NULL if it fails to allocate memory.
   *
   * Example Usage:
   * @snippet digio_example.cpp Calling GetDigIO
   */
  EXTERN_C
    CCAUXDLL_API
    DIGIOHANDLE
    CCAUXDLL_CALLING_CONV
    GetDigIO(void);

  /*!
   * Delete the DigIO object.
   *
   * Supported Platform(s): XL, XM, XS, XA
   *
   * @return -
   *
   * Example Usage:
   * @snippet digio_example.cpp Calling DigIO_release
   */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    DigIO_release(DIGIOHANDLE);

  /*!
   * Get Digital inputs.
   *
   * Supported Platform(s): XL, XM, XS, XA
   *
   * @param status Status of the four digital input pins.
   * Bit0: Digital input 1.
   * Bit1: Digital input 2.
   * Bit2: Digital input 3.
   * Bit3: Digital input 4.
   * Bit 4..7 are always zero.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet digio_example.cpp Calling DigIO_getDigIO
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    DigIO_getDigIO (DIGIOHANDLE, uint8_t* status);

  /*!
   * Set Digital outputs.
   *
   * Supported Platform(s): XA, XS  
   *
   * @param state State of the four digital output pins.
   * Bit0: Digital output 1.
   * Bit1: Digital output 2.
   * Bit2: Digital output 3.
   * Bit3: Digital output 4.
   * Bit 4..7 not used.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet digio_example.cpp Calling DigIO_setDigIO
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    DigIO_setDigIO (DIGIOHANDLE, uint8_t state);

  /*!
  * Get digital power output status.
  *
  * Supported Platform(s): VS  
  *
  * @param output Which output to read
  * @param enabled State of the power output signal.
  * @param status Error status of the power output signal.
  *
  * Error status: The error status byte can be used to detect certain error conditions. If the error status byte is 
  * set to 0, this indicates an error (1 is no error).
  *
  * If the signal state is Enabled (high), the following error conditions gererates an error status:
  * "Short circuit to GND" or "Over temperature". SS monitors the signal and turns it off immediately if 
  * an error is indicated when the signal has been set high. This means that "enabled" will be read as disabled 
  * and in this case the status may have any value - although in most cases it will be set to no error 
  * since the signal is now turned off. 
  *
  * If the signal is disabled (low), these error conditions generates an error status:
  * "Short circuit to VDD" or "Off state open load".
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet digio_example.cpp Calling DigIO_getDigPowerOutput
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    DigIO_getDigPowerOutput(DIGIOHANDLE, PowerOutput output, CCStatus* enabled, uint8_t* status);


  /*!
  * Set Digital power outputs.
  *
  * Supported Platform(s): VS  
  *
  * @param output The output to set
  * @param enabled State of the power output signal.
  * 
  * Outputs 1 and 2 can also be set to be turned on automatically at system startup using:
  * Config_setDigPowerOutputStartupConfig.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet digio_example.cpp Calling DigIO_setDigPowerOutput
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    DigIO_setDigPowerOutput(DIGIOHANDLE, PowerOutput output, CCStatus enabled);


#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_DIGIO_H */

