/************************************************
 *
 *  CROSSCONTROL AUX API
 *
 *  FrontLED.h
 *
 *  Version 2.15.2.0, 2020-07-08
 *
 *  (c) CrossControl 2010-2020
 *

 *  File description: 
 *  Front LED control
 * 
 *  CCpilot VC platform - Button backlight control.
 *  For VC, the API works the same way as for front LED control on other platforms, but without color support.
 *
 *  Use the function GetFrontLED() to get a handle to pass on to the other functions. 
 *  Use the method FrontLED_release(FRONTLEDHANDLE) to return the handle.
 *
 *  Example Usage:
 *  @include frontled_example.cpp
 *
 ************************************************/
#ifndef CC_FRONT_LED_H 
#define CC_FRONT_LED_H

#include "CCAuxErrors.h"
#include "CCAuxTypes.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* FRONTLEDHANDLE;

  /*!
   * Factory function that creates instances of the FrontLED object.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @return FRONTLEDHANDLE to an allocated FrontLED object. The returned
   * handle needs to be deallocated using the FrontLED_release(FRONTLEDHANDLE)
   * method when it's no longer needed.
   * Returns NULL if it fails to allocate memory.
   *
   * Example Usage:
   * @snippet frontled_example.cpp Calling GetFrontLED
   */
  EXTERN_C
    CCAUXDLL_API
    FRONTLEDHANDLE
    CCAUXDLL_CALLING_CONV
    GetFrontLED(void);

  /*!
   * Delete the FrontLED object.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @return -
   *
   * Example Usage:
   * @snippet frontled_example.cpp Calling FrontLED_release    
   */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    FrontLED_release(FRONTLEDHANDLE);

  /*!
   * Get front LED signal. Note, the values may vary from previously set values with setSignal.
   * This is due to precision-loss in approximations.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param frequency LED blink frequency (0.2-50 Hz).
   * @param dutyCycle LED on duty cycle (0-100%).
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet frontled_example.cpp Calling FrontLED_getSignal
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_getSignal (FRONTLEDHANDLE, float64_t* frequency, uint8_t* dutyCycle);

  /*!
   * Get front LED on time.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param onTime Time in 10ms increments. 0 = off
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_getOnTime (FRONTLEDHANDLE, uint8_t* onTime);

  /*!
   * Get front LED off time.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param offTime Time in 10ms increments.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_getOffTime (FRONTLEDHANDLE, uint8_t* offTime);

  /*!
   * Get front LED idle time.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param idleTime Time in 100ms increments.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_getIdleTime (FRONTLEDHANDLE, uint8_t* idleTime);

  /*!
   * Get number of pulses during a blink sequence.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param nrOfPulses Number of pulses.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_getNrOfPulses (FRONTLEDHANDLE, uint8_t* nrOfPulses);

  /*!
   * Get front LED color mix.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * On the VC, VA platforms - the blue parameter gets the button backlight intensity (0-15)
   *
   * @param red Red color intensity 0-0x0F.
   * @param green Green color intensity 0-0x0F.
   * @param blue Blue color intensity 0-0x0F.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet frontled_example.cpp Calling FrontLED_getColor
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_getColor(FRONTLEDHANDLE, uint8_t* red, uint8_t* green, uint8_t* blue);

  /*!
   * Get front LED color from a set of standard colors. 
   * If the color is not one of the predefined colors, UNDEFINED_COLOR will be returned.
   * It is not recommended to use this function on the VC or VA platforms.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param color Color from CCAuxColor enum.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_getStandardColor(FRONTLEDHANDLE, CCAuxColor* color);

  /*!
   * Is the front LED enabled during startup? If enabled, the LED will blink
   * yellow to indicate startup progress. It will turn green once the OS has started.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param status LED Enabled or Disabled during startup.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_getEnabledDuringStartup(FRONTLEDHANDLE, CCStatus* status);

  /*!
   * Set front LED signal.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param frequency LED blink frequency (0.2-50 Hz).
   * @param dutyCycle LED on duty cycle (0-100%).
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Note: The hardware cannot be set to have an on or off time of the LED that's longer than 2.55s (255*10ms)
   * Hence there are limitations in this function when using frequencies slower than 0.39Hz. 
   * At 0.38Hz, the valid duty cycle range is [3 - 97]. 
   * At 0.30Hz, the valid duty cycle range is [24 - 76]. 
   * At 0.20Hz, the valid duty cycle range is [49 - 51]. 
   * At 0.19Hz and slower, the behavior is undefined for all duty cycles, so this is not allowed to be set.
   * Additionally, the hardware cannot be set to have an on or off time of the LED that's shorter than 10ms.
   * Hence, there are limitations in this function when using high frequencies.
   * At 50 Hz, the valid duty cycle range is [50].
   * At 30 Hz, the valid duty cycle range is [30-70].
   * At 10 Hz, the valid duty cycle range is [10-90].
   * At 2 Hz, the valid duty cycle range is [2-98].
   * The behavior is undefined outside these ranges but setting 0% or 100% duty cycle will always work, regardless of the frequency.
   * If you need to blink in an unsupported range, it can be done with a software timer instead. 
   *
   * Example Usage:
   * @snippet frontled_example.cpp Calling FrontLED_setSignal
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_setSignal (FRONTLEDHANDLE, float64_t frequency, uint8_t dutyCycle);

  /*!
   * Set front LED on time.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param onTime Time in 10ms increments. 0 = off
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet frontled_example.cpp Calling FrontLED_setOnTime
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_setOnTime (FRONTLEDHANDLE, uint8_t onTime);

  /*!
   * Set front LED off time.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param offTime Time in 10ms increments.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet frontled_example.cpp Calling FrontLED_setOffTime
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_setOffTime (FRONTLEDHANDLE, uint8_t offTime);

  /*!
   * Get front LED idle time.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param idleTime Time in 100ms.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_setIdleTime (FRONTLEDHANDLE, uint8_t idleTime);

  /*!
   * Set front LED number of pulses during a blink sequence.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param nrOfPulses Number of pulses.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_setNrOfPulses (FRONTLEDHANDLE, uint8_t nrOfPulses);

  /*!
   * Set front LED color mix.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * On the VC, VA platforms - use the blue parameter to set the button backlight intensity (0-15)
   *
   * @param red Red color intensity 0-0x0F.
   * @param green Green color intensity 0-0x0F.
   * @param blue Blue color intensity 0-0x0F.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet frontled_example.cpp Calling FrontLED_setColor
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_setColor(FRONTLEDHANDLE, uint8_t red, uint8_t green, uint8_t blue);

  /*!
   * Set one of the front LED standard colors.
   * It is not recommended to use this function on the VC/VA/VI2 platform.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param color Color from CCAuxColor enum.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet frontled_example.cpp Calling FrontLED_setStandardColor
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_setStandardColor(FRONTLEDHANDLE, CCAuxColor color);

  /*!
   * Set front LED off.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_setOff(FRONTLEDHANDLE);

  /*!
   * Should the front LED be enabled during startup? If enabled, the LED will blink
   * yellow to indicate startup progress. It will turn green once the OS has started.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
   *
   * @param status Enable or Disable the LED during startup.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_setEnabledDuringStartup(FRONTLEDHANDLE, CCStatus status);


  /*!
  * Get LED configuration during system boot.
  * The behavior of the LED in terms of color (intensity) and blink frequency may be configured during system boot.
  * Note that FrontLED_setEnabledDuringStartup overrides these settings, it has to be enabled for these settings to apply.
  * Also note that the frequency limitations mentioned in the description of FrontLED_setSignal also apply here.
  * 
  * Supported Platform(s): VS, VI2, V700
  *
  * @param red Red color intensity 0-0x0F.
  * @param green Green color intensity 0-0x0F.
  * @param blue Blue color intensity 0-0x0F.
  * @param frequency LED blink frequency (0.2-50 Hz).
  * @param dutyCycle LED on duty cycle (0-100%).
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_getBootLEDConfig(FRONTLEDHANDLE, uint8_t* red, uint8_t* green, uint8_t* blue, float32_t* frequency, uint8_t* dutyCycle);

  /*!
  * Set LED configuration during system boot.
  * The behavior of the LED in terms of color (intensity) and blink frequency may be configured during system boot.
  * Note that FrontLED_setEnabledDuringStartup overrides these settings, it has to be enabled for these settings to apply.
  * Also note that the frequency limitations mentioned in the description of FrontLED_setSignal also apply here.
  * 
  * Supported Platform(s): VS, VI2, V700
  *
  * @param red Red color intensity 0-0x0F.
  * @param green Green color intensity 0-0x0F.
  * @param blue Blue color intensity 0-0x0F.
  * @param frequency LED blink frequency (0.2-50 Hz).
  * @param dutyCycle LED on duty cycle (0-100%).
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_setBootLEDConfig(FRONTLEDHANDLE, uint8_t red, uint8_t green, uint8_t blue, float32_t frequency, uint8_t dutyCycle);

  /*!
  * Get LED configuration after system boot.
  * The behavior of the LED in terms of color (intensity) and blink frequency may be configured.
  * The system will set the LED to this state when the operating system has booted. 
  * The LED settings may after this time, at any time, be overridden by other FrontLED functions.
  * Note that FrontLED_setEnabledDuringStartup overrides these settings, it has to be enabled for these settings to apply.
  * Also note that the frequency limitations mentioned in the description of FrontLED_setSignal also apply here.
  * 
  * Supported Platform(s): VS, VI2, V700
  *
  * @param red Red color intensity 0-0x0F.
  * @param green Green color intensity 0-0x0F.
  * @param blue Blue color intensity 0-0x0F.
  * @param frequency LED blink frequency (0.2-50 Hz).
  * @param dutyCycle LED on duty cycle (0-100%).
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_getPostBootLEDConfig(FRONTLEDHANDLE, uint8_t* red, uint8_t* green, uint8_t* blue, float32_t* frequency, uint8_t* dutyCycle);

  /*!
  * Set LED configuration after system boot.
  * The behavior of the LED in terms of color (intensity) and blink frequency may be configured.
  * The system will set the LED to this state when the operating system has booted. 
  * The LED settings may after this time, at any time, be overridden by other FrontLED functions.
  * Note that FrontLED_setEnabledDuringStartup overrides these settings, it has to be enabled for these settings to apply.
  * Also note that the frequency limitations mentioned in the description of FrontLED_setSignal also apply here.
  * 
  * Supported Platform(s): VS, VI2, V700
  *
  * @param red Red color intensity 0-0x0F.
  * @param green Green color intensity 0-0x0F.
  * @param blue Blue color intensity 0-0x0F.
  * @param frequency LED blink frequency (0.2-50 Hz).
  * @param dutyCycle LED on duty cycle (0-100%).
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_setPostBootLEDConfig(FRONTLEDHANDLE, uint8_t red, uint8_t green, uint8_t blue, float32_t frequency, uint8_t dutyCycle);



#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#ifdef PLATFORM_VI2
#pragma message "FrontLED is deprecated for CCpilot VI2"
#endif

#endif /* CC_FRONT_LED_H */

