/************************************************
 *
 *  CROSSCONTROL AUX API
 *
 *  Lightsensor.h
 *
 *  Version 2.15.2.0, 2020-07-08
 *
 *  (c) CrossControl 2010-2020
 *

 *  File description: 
 *  Light Sensor access. 
 *  The lightsensor suported by this implementation is a TAOS TSL2550 for 
 *  platforms other than VC.
 *  On the VC platform, the sensor is the Intersil ISL29035.
 * 
 *  Note that reading the light sensor using average calculation at the 
 *  same time as running the automatic backlight control is not supported.
 *  Also note that Lux values mentioned below (and in the Backlight class) 
 *  are not necessarily true lux values. The values received are lower than
 *  true lux values, due to the light guide in the front panel, where some 
 *  light is lost. It is still a measurement of the illuminance (in Lux).
 *
 *  Use the function GetLightsensor() to get a handle to pass on to the other functions. 
 *  Use the method Lightsensor_release(LIGHTSENSORHANDLE) to return the handle.
 *
 *  Example Usage:
 *  @include lightsensor_example.cpp
 *
 ************************************************/
#ifndef CC_LIGHTSENSOR_H 
#define CC_LIGHTSENSOR_H

#include "CCAuxErrors.h"
#include "CCAuxTypes.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* LIGHTSENSORHANDLE;

  /*!
   * Factory function that creates instances of the Lightsensor object.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, V700
   *
   * @return LIGHTSENSORHANDLE to an allocated Lightsensor object. The returned
   * handle needs to be deallocated using the Lightsensor_release(LIGHTSENSORHANDLE) 
   * method when it's no longer needed.
   * Returns NULL if it fails to allocate memory.
   *
   * Example Usage:
   * @snippet lightsensor_example.cpp Calling GetLightsensor
   */
  EXTERN_C
    CCAUXDLL_API
    LIGHTSENSORHANDLE
    CCAUXDLL_CALLING_CONV
    GetLightsensor(void);

  /*!
   * Delete the Lightsensor object.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, V700
   *
   * @return -
   *
   * Example Usage:
   * @snippet lightsensor_example.cpp Calling Lightsensor_release
   */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    Lightsensor_release(LIGHTSENSORHANDLE);

  /*!
   * Get illuminance (light) value from light sensor.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, V700
   *
   * @param value Illuminace value (Lux).
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet lightsensor_example.cpp Calling Lightsensor_getIlluminance
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Lightsensor_getIlluminance (LIGHTSENSORHANDLE, uint16_t* value);

  /*!
   * Get illuminance (light) value from light sensor.
   * The parameters cho and ch1 are raw ADC values read from a TAOS TSL2550 lightsensor. 
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, V700
   *
   * @param value Illuminance value (Lux).
   * @param ch0 Channel0 value. (Not applicable on VC platform - always 0)
   * @param ch1 Channel1 value. (Not applicable on VC platform - always 0)
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Lightsensor_getIlluminance2 (LIGHTSENSORHANDLE, uint16_t* value, uint8_t* ch0, uint8_t* ch1);

  /*!
   * Get average illuminance (light) value from light sensor.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, V700
   *
   * @param value Illuminance value (Lux).
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet lightsensor_example.cpp Calling Lightsensor_getAverageIlluminance
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Lightsensor_getAverageIlluminance (LIGHTSENSORHANDLE, uint16_t* value);

  /*!
   * Start average calculation.
   * The average calculation works by calculating the average from a number of consecutive samples, the average window size. 
   * The reject window is used to discard sudden changes or single extreme values of the measurement. If the difference of the maximum 
   * value and the minimum value in the number of samples in the reject delta window is larger than the reject delta, those samples are discarded.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, V700
   *
   * @param averageWndSize The average window size in nr of samples.
   * @param rejectWndSize The reject window size in nr of samples.
   * @param rejectDeltaInLux The reject delta in lux.
   * @param mode The configured sampling mode.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet lightsensor_example.cpp Calling Lightsensor_startAverageCalc
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Lightsensor_startAverageCalc(LIGHTSENSORHANDLE, 
				 uint32_t averageWndSize,
				 uint32_t rejectWndSize,
				 uint32_t rejectDeltaInLux,
				 LightSensorSamplingMode mode);

  /*!
   * Stop average calculation.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, V700
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet lightsensor_example.cpp Calling Lightsensor_stopAverageCalc
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Lightsensor_stopAverageCalc(LIGHTSENSORHANDLE);

  /*!
   * Get operating range. The light sensor can operate in two ranges. Standard and extended range. In standard range,
   * the range is smaller but resolution higher. See the TSL2550 data sheet for more information.
   * On the VC platform, the ranges correspond to 1000 and 4000 lux maximum value.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, V700
   * 
   * @param range	Operating range. RangeStandard or RangeExtended.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Lightsensor_getOperatingRange(LIGHTSENSORHANDLE, LightSensorOperationRange* range);

  /*!
   * Set operating range. The light sensor can operate in two ranges. Standard and extended range. In standard range,
   * the range is smaller but resolution higher. See the TSL2550 data sheet for more information.
   * On the VC platform, the ranges correspond to 1000 and 4000 lux maximum value.
   *
   * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, V700
   *
   * @param range Operating range to set. RangeStandard or RangeExtended.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Lightsensor_setOperatingRange(LIGHTSENSORHANDLE, LightSensorOperationRange range);

#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_LIGHTSENSOR_H */

