/************************************************
 *
 *  CROSSCONTROL AUX API
 *
 *  PWMOut.h
 *
 *  Version 2.15.2.0, 2020-07-08
 *
 *  (c) CrossControl 2010-2020
 *

 *  File description: 
 *  PWM Output
 *
 *  Use the function GetPWMOut() to get a handle to pass on to the other functions.
 *  Use the method PWMOut_release(PWMOUTHANDLE) to return the handle.
 *
 *  Note: The rise and fall times of the hardware limits the signal's usefulness in high frequencies.
 *  The duty cycle of the PWM signal affects the maximum attainable frequency: at 50% duty cycle, the attainable frequecy is at it's maximum.
 *  If the duty cycle is near 0 or 100%, the rise/fall times of the signal means that the maximum attainable frequency is significantly lower.
 *  A PWM signal is generated when both the frequency and the duty cycle of a channel is not zero.
 *
 *  Example Usage:
 *  @include pwmout_example.cpp
 *
 ************************************************/
#ifndef CC_PWMOUT_H
#define CC_PWMOUT_H

#include "CCAuxErrors.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* PWMOUTHANDLE;

  /*!
  * Factory function that creates instances of the PWMOut object.
  *
  * Supported Platform(s): VC, VA, VI2
  *
  * @return PWMOUTHANDLE to an allocated PWMOut object. The returned
  * handle needs to be deallocated using the PWMOut_release(PWMOUTHANDLE)
  * method when it's no longer needed.
  * Returns NULL if it fails to allocate memory.
  *
  * Example Usage:
  * @snippet pwmout_example.cpp Calling GetPWMOut
  */
  EXTERN_C
    CCAUXDLL_API
    PWMOUTHANDLE
    CCAUXDLL_CALLING_CONV
    GetPWMOut(void);

  /*!
  * Delete the PWMOut object.
  *
  * Supported Platform(s): VC, VA, VI2
  *
  * @return -
  *
  * Example Usage:
  * @snippet pwmout_example.cpp Calling PWMOut_release
  */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    PWMOut_release(PWMOUTHANDLE);

  /*!
  * Set PWM Output Duty cycle for a channel
  *
  * Supported Platform(s): VC, VA, VI2
  *
  * @param channel Which channel to set
  * There are two output channels, 1 or 2.
  * @param duty_cycle Which duty cycle (0-100 %) to use
  * NOTE: For low side PWM outputs, a duty cycle of 60% means 60% low, 40% high
  *       For high side PWM outputs, a duty cycle of 60% means 60% high, 40% low
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet pwmout_example.cpp Calling PWMOut_setPWMOutputChannelDutyCycle
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    PWMOut_setPWMOutputChannelDutyCycle (PWMOUTHANDLE, uint8_t channel, uint8_t duty_cycle);

  /*!
  * Set PWM Output frequency for a channel
  *
  * Supported Platform(s): VC, VA, VI2
  *
  * @param channel Which channel to set
  * There are two output channels, 1 or 2.
  * @param frequency 0.0 - 5000.0 Hz frequency value
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet pwmout_example.cpp Calling PWMOut_setPWMOutputChannelFrequency
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    PWMOut_setPWMOutputChannelFrequency (PWMOUTHANDLE, uint8_t channel, float32_t frequency);

  /*!
  * Get PWM Output channel duty cycle
  *
  * Supported Platform(s): VC, VA, VI2
  *
  * @param channel Which channel to get value from
  * There are two output channels, 1 or 2.
  * @param duty_cycle The read back duty cycle value
  * NOTE: For low side PWM outputs, a duty cycle of 60% means 60% low, 40% high
  *       For high side PWM outputs, a duty cycle of 60% means 60% high, 40% low
  * 
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet pwmout_example.cpp Calling PWMOut_getPWMOutputChannelDutyCycle
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    PWMOut_getPWMOutputChannelDutyCycle (PWMOUTHANDLE, uint8_t channel, uint8_t* duty_cycle);

  /*!
  * Get PWM Output frequency for a channel
  *
  * Supported Platform(s): VC, VA, VI2
  *
  * @param channel Which channel to set
  * There are two output channels, 1 or 2.
  * @param frequency 0.0 - 5000.0 Hz frequency value
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet pwmout_example.cpp Calling PWMOut_getPWMOutputChannelFrequency
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    PWMOut_getPWMOutputChannelFrequency (PWMOUTHANDLE, uint8_t channel, float32_t* frequency);

  /*!
  * Get PWM Output status
  *
  * Supported Platform(s): VC, VA, VI2
  *
  * @param status Read back status value
  * Bit 0 represents PWM Output channel 1.
  * Bit 1 represents PWM Output channel 2.
  * If bit is set, it means unconnected, short to ground or over temperature detected. The output will be turned off when the error occurrs.
  * The error status remains until the output is turned on successfully. 
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet pwmout_example.cpp Calling PWMOut_getPWMOutputStatus
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    PWMOut_getPWMOutputStatus (PWMOUTHANDLE, uint8_t* status);

  /*!
  * Turn off a PWM Output channel.
  * This function sets both frequency and duty cycle to 0. 
  *
  * Supported Platform(s): VC, VA, VI2
  *
  * @param channel Which channel to set
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet pwmout_example.cpp Calling PWMOut_setPWMOutOff
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    PWMOut_setPWMOutOff (PWMOUTHANDLE, uint8_t channel);

#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_PWMOUT_H */

