/************************************************
 *
 *  CROSSCONTROL AUX API
 *
 *  PowerMgr.h
 *
 *  Version 2.15.2.0, 2020-07-08
 *
 *  (c) CrossControl 2010-2020
 *

 *  File description: 
 *  Access functions for managing application shutdown/cleanup 
 *  when suspend or shutdown events occur. An application can use this framework 
 *  to delay a suspend/shutdown until it is ready with any tasks it needs to do.
 *  It is also possible to use existing OS functions to achieve the same result.
 *
 *  Note that applications should not use these functions to delay a suspend/shutdown 
 *  request for a long time (up to 20s may be okay). The computer will eventually 
 *  be shut down forcibly if the application does not respond. 
 *
 *  It is not recommended for multiple applications to access the PowerMgr
 *  functionality simultaneously, although it is supported on some platforms.
 *
 *  Note that a shutdown initiated by the OS will not be caught by this framework. 
 *
 *  Example Usage:
 *  @include powermgr_example.cpp
 *
 ************************************************/

#ifndef CC_POWERMGR_H_
#define CC_POWERMGR_H_

#include "CCAuxErrors.h"
#include "CCAuxTypes.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl
{
#endif

  /*!
  * Enumeration of the settings that can be used with the PowerMgr system.
  */
  typedef enum
  {
    /*! 
    * Applications will not be able to delay suspend/shutdown requests. 
    * This is the normal configuration that is used when the PowerMgr class is not being used.
    * Setting this configuration turns off the feature if it is in use.
    */
    Normal = 0,

    /*!
    * Applications can delay suspend/shutdown requests.
    */
    ApplicationControlled = 1, 

    /*! 
    * In this mode, the computer will automatically enter suspend mode when the unit starts running 
    * on battery power. Applications can delay suspend/shutdown requests.
    * This mode is only applicable if the unit has an external battery. Using this configuration 
    * on a computer without an external battery will be the same as using the configuration ApplicationControlled.
    */
    BatterySuspend = 2
  } PowerMgrConf;

  typedef enum 
  {
    /*!
    * No suspend or shutdown requests.
    */
    NoRequestsPending = 0,
    /*!
    * A suspend request is pending.
    */
    SuspendPending = 1,
    /*!
    * A shutdown request is pending.
    */
    ShutdownPending = 2
  }PowerMgrStatus;

  typedef void* POWERMGRHANDLE;

  /*!
  * Factory function that creates instances of the PowerMgr object.
  *
  * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
  *   
  * @return POWERMGRHANDLE to an allocated PowerMgr structure. The returned
  * handle needs to be deallocated using the PowerMgr::Release() method when it's
  * no longer needed.
  * Returns NULL if it fails to allocate memory.
  *
  * Example Usage:
  * @snippet powermgr_example.cpp Calling GetPowerMgr
  */
  EXTERN_C
    CCAUXDLL_API
    POWERMGRHANDLE
    CCAUXDLL_CALLING_CONV
    GetPowerMgr(void);

  /*!
  * Delete the PowerMgr object.
  *
  * Supported Platform(s): XL, XL5, XM, XM9, XS, XA, VC, VA, VS, VI2, V700
  *
  * @return -
  *
  * Example Usage:
  * @snippet powermgr_example.cpp Calling Release
  */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    PowerMgr_release(POWERMGRHANDLE);

  /*!
  * Configure the PowerMgr. Call this function once initially to turn on the functionality. 
  *
  * Supported Platform(s): XL, XL5 ,XM, XM9, XS, XA, VC, VA, VS, VI2, V700
  *
  * @param conf The configuration to use.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet powermgr_example.cpp Calling registerControlledSuspendOrShutDown
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    PowerMgr_registerControlledSuspendOrShutDown(POWERMGRHANDLE, PowerMgrConf conf);

  /*!
  * Get the configuration that is in use. 
  *
  * Supported Platform(s): XL, XL5 ,XM, XM9, XS, XA, VC, VA, VS, VI2, V700
  *
  * @param conf The configuration in use.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet powermgr_example.cpp Calling getConfiguration
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    PowerMgr_getConfiguration(POWERMGRHANDLE, PowerMgrConf* conf);

  /*!
  * Get the current status of the PowerMgr. This functions should be called periodically, 
  * to detect when suspend or shutdown requests arrive. 
  *
  * Supported Platform(s): XL, XL5 ,XM, XM9, XS, XA, VC, VA, VS, VI2, V700
  *
  * @param status The current status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet powermgr_example.cpp Calling getPowerMgrStatus
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    PowerMgr_getPowerMgrStatus(POWERMGRHANDLE, PowerMgrStatus* status);

  /*!
  * Acknowledge that the application is ready for suspend/shutdown. Should be called after a request has been received in order to execute the request.
  * The application must acknowledge a request within 20s from when it arrives.
  *
  * Supported Platform(s): XL, XL5 ,XM, XM9, XS, XA, VC, VA, VS, VI2, V700
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet powermgr_example.cpp Calling setAppReadyForSuspendOrShutdown
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    PowerMgr_setAppReadyForSuspendOrShutdown(POWERMGRHANDLE);

  /*!
  * This function can be used in a suspend-resume scenario. After the application has used setAppReadyForSuspendOrShutdown() to init the suspend, 
  * this function may be polled in order to detect when the system is up and running again. 
  * Calling this function before calling setAppReadyForSuspendOrShutdown will return resumed = true.
  *
  * Supported Platform(s): XL, XL5, XM, XM9 (CC Linux), VC, VA, VS, VI2, V700
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet powermgr_example.cpp Calling hasResumed
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    PowerMgr_hasResumed(POWERMGRHANDLE, bool* resumed);

#ifdef __cplusplus
}
#endif

#endif /* CC_POWERMGR_H_ */


