/************************************************
 *
 *  CROSSCONTROL AUX API
 *
 *  SoftKey.h
 *
 *  Version 2.15.2.0, 2020-07-08
 *
 *  (c) CrossControl 2010-2020
 *

 *  File description:
 *  Get current status of buttons connected to SS
 *
 *  Use the function GetSoftKey() to get a handle to pass on to the other functions.
 *  Use the method SoftKey_release(SOFTKEYHANDLE) to return the handle.
 *
 *  Example Usage:
 *  @include softkey_example.cpp
 *
 ************************************************/

#ifndef CC_SOFTKEY_H_
#define CC_SOFTKEY_H_

#include "CCAuxErrors.h"
#include "CCPlatform.h"
#include "CCAuxTypes.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* SOFTKEYHANDLE;

  /*!
   * Factory function that creates instances of the SoftKey object.
   *
   * Supported Platform(s): VI2
   *
   * @return SOFTKEYHANDLE to an allocated SoftKey object. The returned
   * handle needs to be deallocated using the SoftKey_release(SOFTKEYHANDLE) method when it's
   * no longer needed.
   *
   * Returns NULL if it fails to allocate memory.
   *
   * Example Usage:
   * @snippet softkey_example.cpp Calling GetSoftKey
   */
  EXTERN_C
    CCAUXDLL_API
    SOFTKEYHANDLE
    CCAUXDLL_CALLING_CONV
    GetSoftKey(void);

  /*!
   * Delete the SOFTKEY object.
   *
   * Supported Platform(s): VI2
   *
   * @return -
   *
   * Example Usage:
   * @snippet sotfkey_example.cpp Calling SoftKey_release
   */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    SoftKey_release(SOFTKEYHANDLE);

  /*!
   * Get current status of keys
   *
   * Supported Platform(s): VI2
   *
   * @param value Bitfield of keys that are pressed. LSB is KEY1.
   * Can be undefined if return value is error code.
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet softkey_example.cpp Calling SoftKey_getStatus
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    SoftKey_getStatus(SOFTKEYHANDLE, uint16_t* value);

  /*!
   * Set softkey backlight intensity.
   *
   * Supported Platform(s): VI2
   *
   * @param key Key that's intensity is set. Start index 1.
   * @param intensity Intensity value in range 0 - 100%
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet softkey_example.cpp Calling SoftKey_setBacklightIntensity
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    SoftKey_setBacklightIntensity(SOFTKEYHANDLE, uint8_t key, uint8_t intensity);

  /*!
   * Get softkey backlight intensity.
   *
   * Supported Platform(s): VI2
   *
   * @param key Key that's intensity is read. Start index 1.
   * @param intensity Intensity value 0 - 100%
   *
   * Due limitation in HW, readback value may differ from the value that was set.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet softkey_example.cpp Calling SoftKey_getBacklightIntensity
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    SoftKey_getBacklightIntensity(SOFTKEYHANDLE, uint8_t key, uint8_t *intensity);

  /*!
   * Set Multiple SoftKey backlight intensities.
   *
   * Supported Platform(s): VI2
   *
   * @param keys Array of Keys that's intensity to set. Key indexing starts at number 1.
   * @param intensities Array of intensities for the keys listed above. Intensity value in range 0 - 100%
   * @param array_size Number of key/intensity - pairs
   *
   * To set all keys to same intensitity. Use key number 0 and array_size 1. 
   *
   * Not all values are supported on all platforms, ERR_NOT_SUPPORTED will indicate that.
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet softkey_example.cpp Calling SoftKey_setMultipleBacklightIntensities
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    SoftKey_setMultipleBacklightIntensities(SOFTKEYHANDLE, uint8_t* keys, uint8_t* intensities, uint8_t array_size);

  /*!
   * Get multiple SoftKey backlight intensities.
   *
   * Supported Platform(s): VI2
   *
   * @param keys Array of Keys that's intensity to set. Key indexing starts at number 1.
   * @param intensities Array of intensities for the keys listed above. Inetnsity value in range 0 - 100%
   * @param array_size Number of key/intensity - pairs
   *
   * Due limitation in HW, readback value may differ from the value that was set.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet softkey_example.cpp Calling SoftKey_getMultipleBacklightIntensities
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    SoftKey_getMultipleBacklightIntensities(SOFTKEYHANDLE, uint8_t* keys, uint8_t *intensities, uint8_t array_size);

  /*!
   * Get SoftKey Backlight signal. Note, the values may vary from previously set values with setSignal.
   * This is due to precision-loss in approximations.
   *
   * Supported Platform(s): VI2
   *
   * @param frequency Backlight blink frequency (0.2-50 Hz).
   * @param dutyCycle Backlight on duty cycle (0-100%).
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet softkey_example.cpp Calling SoftKey_getBacklightSignal
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    SoftKey_getBacklightSignal (SOFTKEYHANDLE, float64_t* frequency, uint8_t* dutyCycle);

  /*!
   * Get SoftKey Backlight ON time.
   *
   * Supported Platform(s): VI2
   *
   * @param onTime Time in 10ms increments. 0 = off
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    SoftKey_getBacklightOnTime (SOFTKEYHANDLE, uint8_t* onTime);

  /*!
   * Get SoftKey Backlight OFF time.
   *
   * Supported Platform(s): VI2
   *
   * @param offTime Time in 10ms increments.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    SoftKey_getBacklightOffTime (SOFTKEYHANDLE, uint8_t* offTime);

  /*!
   * Get SoftKey Backlight IDLE time.
   *
   * Supported Platform(s): VI2
   *
   * @param idleTime Time in 100ms increments.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    SoftKey_getBacklightIdleTime (SOFTKEYHANDLE, uint8_t* idleTime);

  /*!
   * Get number of pulses during a Softkey backlight blink sequence.
   *
   * Supported Platform(s): VI2
   *
   * @param nrOfPulses Number of pulses.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    SoftKey_getBacklightNrOfPulses (SOFTKEYHANDLE, uint8_t* nrOfPulses);

  /*!
   * Set SoftKey Backlight signal.
   *
   * Supported Platform(s): VI2
   *
   * @param frequency SoftKey Backlight blink frequency (0.2-50 Hz).
   * @param dutyCycle SoftKey Backlight ON duty cycle (0-100%).
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Note: The hardware cannot be set to have an on or off time of the LED that's longer than 2.55s (255*10ms)
   * Hence there are limitations in this function when using frequencies slower than 0.39Hz. 
   * At 0.38Hz, the valid duty cycle range is [3 - 97]. 
   * At 0.30Hz, the valid duty cycle range is [24 - 76]. 
   * At 0.20Hz, the valid duty cycle range is [49 - 51]. 
   * At 0.19Hz and slower, the behavior is undefined for all duty cycles, so this is not allowed to be set.
   * Additionally, the hardware cannot be set to have an on or off time of the LED that's shorter than 10ms.
   * Hence, there are limitations in this function when using high frequencies.
   * At 50 Hz, the valid duty cycle range is [50].
   * At 30 Hz, the valid duty cycle range is [30-70].
   * At 10 Hz, the valid duty cycle range is [10-90].
   * At 2 Hz, the valid duty cycle range is [2-98].
   * The behavior is undefined outside these ranges but setting 0% or 100% duty cycle will always work, regardless of the frequency.
   * If you need to blink in an unsupported range, it can be done with a software timer instead. 
   *
   * Example Usage:
   * @snippet softkey_example.cpp Calling SoftKey_setBacklightSignal
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    SoftKey_setBacklightSignal (SOFTKEYHANDLE, float64_t frequency, uint8_t dutyCycle);

  /*!
   * Set SoftKey Backlight ON time.
   *
   * Supported Platform(s): VI2
   *
   * @param onTime Time in 10ms increments. 0 = off
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet softkey_example.cpp Calling SoftKey_setBackglightOnTime
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    SoftKey_setBacklightOnTime (SOFTKEYHANDLE, uint8_t onTime);

  /*!
   * Set SoftKey Backlight OFF time.
   *
   * Supported Platform(s): VI2
   *
   * @param offTime Time in 10ms increments.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet softkey_example.cpp Calling SoftKey_setBacklightOffTime
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    SoftKey_setBacklightOffTime (SOFTKEYHANDLE, uint8_t offTime);

  /*!
   * Set SoftKey Backlight IDLE time.
   *
   * Supported Platform(s): VI2
   *
   * @param idleTime Time in 100ms.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    SoftKey_setBacklightIdleTime (SOFTKEYHANDLE, uint8_t idleTime);

  /*!
   * Set number of pulses during a SoftKey Backlight blink sequence.
   *
   * Supported Platform(s): VI2
   *
   * @param nrOfPulses Number of pulses.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    SoftKey_setBacklightNrOfPulses (SOFTKEYHANDLE, uint8_t nrOfPulses);

  /*!
   * Set Softkey Backlight off.
   *
   * Supported Platform(s): VI2
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    SoftKey_setBacklightOff(SOFTKEYHANDLE);

  /*!
   * Is the Softkey Backlight enabled during startup? If enabled, the LED will blink
   * to indicate startup progress. It will turn solid once the OS has started.
   *
   * Supported Platform(s): VI2
   *
   * @param status Softkey Backlight Enabled or Disabled during startup.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    SoftKey_getBacklightEnabledDuringStartup(SOFTKEYHANDLE, CCStatus* status);

  /*!
   * Should the Softkey Backlight be enabled during startup? If enabled, the LED will blink
   * to indicate startup progress. It will turn solid once the OS has started.
   *
   * Supported Platform(s): VI2
   *
   * @param status Enable or Disable the Softkey Backlight during startup.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    SoftKey_setBacklightEnabledDuringStartup(SOFTKEYHANDLE, CCStatus status);

  /*!
   * Get SoftKey backlight configuration during and after system boot. These settings applies to all SoftKeys, they cannot be configured individually.
   * The behavior of the backlight LED in terms of intensity and blink frequency may be configured for system boot and after boot. 
   * The post-boot setting can also be disabled using the postBootConfig parameter. When disabled, the boot-settings will remain 
   * until a user application reconfigures the backlight.
   * 
   * Supported Platform(s): VI2
   *
   * @param bootIntensity Backlight intensity (0-100%) during system boot.
   * @param bootFrequency Backlight blink frequency (0.2-50 Hz) during system boot.
   * @param bootDutyCycle Backlight on duty cycle (0-100%) during system boot.
   * @param postBootIntensity Backlight intensity (0-100%) after system boot.
   * @param postBootFrequency Backlight blink frequency (0.2-50 Hz) after system boot.
   * @param postBootDutyCycle Backlight on duty cycle (0-100%) after system boot.
   * @param postBootConfig Post-boot settings enabled or disabled.
   *
   * Due limitation in HW, readback value may differ from the value that was set.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    SoftKey_getBootBacklightConfig(SOFTKEYHANDLE,
				   uint8_t* bootIntensity,
				   float32_t* bootFrequency,
				   uint8_t* bootDutyCycle,
				   uint8_t* postBootIntensity,
				   float32_t* postBootFrequency,
				   uint8_t* postBootDutyCycle,
				   CCStatus* postBootConfig
				   );

  /*!
   * Set SoftKey backlight configuration during and after system boot. These settings applies to all SoftKeys, they cannot be configured individually.
   * The behavior of the backlight LED in terms of intensity and blink frequency may be configured for system boot and after boot. 
   * The post-boot setting can also be disabled using the postBootConfig parameter. When disabled, the boot-settings will remain 
   * until a user application reconfigures the backlight.
   * Note that the frequency limitations mentioned in the description of SoftKey_setBacklightSignal also apply here.
   * 
   * Supported Platform(s): VI2
   *
   * @param bootIntensity Backlight intensity (0-100%) during system boot.
   * @param bootFrequency Backlight blink frequency (0.2-50 Hz) during system boot.
   * @param bootDutyCycle Backlight on duty cycle (0-100%) during system boot.
   * @param postBootIntensity Backlight intensity (0-100%) after system boot.
   * @param postBootFrequency Backlight blink frequency (0.2-50 Hz) after system boot.
   * @param postBootDutyCycle Backlight on duty cycle (0-100%) after system boot.
   * @param postBootConfig Post-boot settings enabled or disabled.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    SoftKey_setBootBacklightConfig(SOFTKEYHANDLE,
				   uint8_t bootIntensity,
				   float32_t bootFrequency,
				   uint8_t bootDutyCycle,
				   uint8_t postBootIntensity,
				   float32_t postBootFrequency,
				   uint8_t postBootDutyCycle,
				   CCStatus postBootConfig
				   );

#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_SOFTKEY_H_ */

