/************************************************
 *
 *  CROSSCONTROL AUX API
 *
 *  TouchScreen.h
 *
 *  Version 2.15.2.0, 2020-07-08
 *
 *  (c) CrossControl 2010-2020
 *

 *  File description: 
 *  Touch Screen settings
 *
 *  Use the function GetTouchScreen() to get a handle to pass on to the other functions. 
 *  Use the method TouchScreen_release(TOUCHSCREENHANDLE) to return the handle.
 *  Note that the VC platform uses a different touch screen solution which is not supported by this API.
 *
 *  Example Usage:
 *  @include touchscreen_example.cpp
 *
 ************************************************/

#ifndef CC_TOUCHSCR_H
#define CC_TOUCHSCR_H

#include "CCAuxErrors.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* TOUCHSCREENHANDLE;

  /*! Touch screen USB profile settings */
  typedef enum
  {
    MOUSE_NEXT_BOOT = 0,  /*! Set the touch USB profile to mouse profile. Active upon the next boot. */
    TOUCH_NEXT_BOOT = 1,  /*! Set the touch USB profile to touch profile. Active upon the next boot. */
    MOUSE_NOW = 2,  /*! Immediately set the touch USB profile to mouse profile. */
    TOUCH_NOW = 3   /*! Immediately set the touch USB profile to touch profile. */
  } TouchScreenModeSettings;


  /*!
  * Fixes spelling issue from previous releases
  */
#define TS_CALIBTATION_WIDTH TS_CALIBRATION_WIDTH 

  /*! 
  * Touch screen advanced settings parameters 
  */
  typedef enum
  {
    /*! 
     * Right click time in ms, except for touch profile on XM platform 
     */
    TS_RIGHT_CLICK_TIME = 0,	

    /*!
     * Lowest A/D value required for registering a touch event. 
     * Front uc 0.5.3.1 had the default value of 3300, newer versions: 3400. 
     */
    TS_LOW_LEVEL = 1,

    /*! 
     * A/D value where the screen is considered to be untouched. 
     */
    TS_UNTOUCHLEVEL = 2,	

    /*! 
     * Debounce time is the time after first detected touch event during which no measurements are being taken.
     * This is used to avoid faulty measurements that frequently happens right after the actual touch event. 
     * Front uc 0.5.3.1 had the default value of 3ms, newer versions: 24ms. 
     */
    TS_DEBOUNCE_TIME = 3,	

    /*! 
     * After debounce, an event will be ignored if after this time there are no valid measurements 
     * above TS_LOW_LEVEL. This time must be larger than TS_DEBOUNCE_TIME. 
     * Front uc 0.5.3.1 had the default value of 12ms, newer versions: 36ms. 
     */
    TS_DEBOUNCE_TIMEOUT_TIME = 4,	

    /*! 
     * Parameter used for improving double click accuracy. A touch event this long or shorter is considered to 
     * be one of the clicks in a double click.
     */
    TS_DOUBLECLICK_MAX_CLICK_TIME = 5,	

    /*! 
     * Parameter used for improving double click accuracy.
     * Time allowed between double clicks. Used for double click improvement.
     */
    TS_DOUBLE_CLICK_TIME = 6, 

    /*! 
     * Maximum distance allowed to move pointer and still consider the event a right click.
     */
    TS_MAX_RIGHTCLICK_DISTANCE = 7,

    /*! 
     * The dejitter function enables smoother pointer movement.
     * Set to non-zero to enable the function or zero to disable it.
     */
    TS_USE_DEJITTER = 8,

    /*! 
     * Accepted difference in measurement during calibration of a point.
     */
    TS_CALIBRATION_WIDTH = 9,

    /*! 
     * Number of measurements needed to accept a calibration point.
     */
    TS_CALIBRATION_MEASUREMENTS = 10, 

    /*!
     * Set to non-zero to restore all the above settings to their defaults. 
     * This parameter cannot be read and setting it to zero has no effect.
     */
    TS_RESTORE_DEFAULT_SETTINGS = 11,
    
    /*!
     * Time (in units of 200 ms) until the touch screen is recalibrated
     * when continuously touching the screen at one point. A setting of 
     * zero disables the recalibration. Valid for PCAP touch panels only.
     * Device must be restarted for changes to have any effect.
     * The default value is 50 which corresponds to 10 seconds.
     */
    TS_TCHAUTOCAL = 12

  } TSAdvancedSettingsParameter;

  /*!
   * Factory function that creates instances of the TouchScreen object.
   *
   * Supported Platform(s): XL, XM, XS, XA  
   *
   * @return TOUCHSCREENHANDLE to an allocated TouchScreen object. The returned
   * handle needs to be deallocated using the TouchScreen_release(TOUCHSCREENHANDLE) 
   * method when it's no longer needed.
   * Returns NULL if it fails to allocate memory.
   *
   * Example Usage:
   * @snippet touchscreen_example.cpp Calling GetTouchScreen
   */
  EXTERN_C
    CCAUXDLL_API
    TOUCHSCREENHANDLE
    CCAUXDLL_CALLING_CONV
    GetTouchScreen(void);

  /*!
   * Delete the TouchScreen object.
   *
   * Supported Platform(s): XL, XM, XS, XA
   *
   * @return -
   *
   * Example Usage:
   * @snippet touchscreen_example.cpp Calling TouchScreen_release
   */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    TouchScreen_release(TOUCHSCREENHANDLE);

  /*!
   * Get Touch Screen mode. Gets the current mode of the USB profile.
   *
   * Supported Platform(s): XL, XM, XS, XA
   *
   * @param config The current mode.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet touchscreen_example.cpp Calling TouchScreen_getMode
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    TouchScreen_getMode(TOUCHSCREENHANDLE, TouchScreenModeSettings* config);

  /*!
   * Get mouse right click time. Applies only to the mouse profile. Use the OS settings for the touch profile.
   *
   * Supported Platform(s): XL, XM, XS, XA 
   *
   * @param time The right click time, in milliseconds.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet touchscreen_example.cpp Calling TouchScreen_getMouseRightClickTime
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    TouchScreen_getMouseRightClickTime(TOUCHSCREENHANDLE, uint16_t* time);

  /*!
   * Set Touch Screen mode. Sets the mode of the USB profile.
   *
   * Supported Platform(s): XL, XM, XS, XA
   *
   * @param config The mode to set.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    TouchScreen_setMode(TOUCHSCREENHANDLE, TouchScreenModeSettings config);

  /*!
   * Set mouse right click time. Applies only to the mouse profile. Use the OS settings for the touch profile.
   *
   * Supported Platform(s): XL, XM, XS, XA
   *
   * @param time The right click time, in milliseconds.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    TouchScreen_setMouseRightClickTime(TOUCHSCREENHANDLE, uint16_t time);

  /*!
   * Set advanced touch screen settings.
   * See the description of TSAdvancedSettingsParameter for a description of the parameters.
   *
   * Supported Platform(s): XL, XM, XS, XA
   *
   * @param param The setting to set.
   * @param data The data value to set. 
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    TouchScreen_setAdvancedSetting(TOUCHSCREENHANDLE, TSAdvancedSettingsParameter param, uint16_t data);

  /*!
   * Get advanced touch screen settings.
   * See the description of TSAdvancedSettingsParameter for a description of the parameters.
   *
   * Supported Platform(s): XL, XM, XS, XA
   *
   * @param param The setting to get.
   * @param data The current data for the setting.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   *
   * Example Usage:
   * @snippet touchscreen_example.cpp Calling TouchScreen_getAdvancedSetting
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    TouchScreen_getAdvancedSetting(TOUCHSCREENHANDLE, TSAdvancedSettingsParameter param, uint16_t* data);

#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_TOUCHSCR_H */

