/************************************************
 *
 *  CROSSCONTROL AUX API
 *
 *  TouchScreenCalib.h
 *
 *  Version 2.15.2.0, 2020-07-08
 *
 *  (c) CrossControl 2010-2020
 *

 *  File description: 
 *  Touch Screen Calibration interface
 *  Note that the VC platform uses a different touch screen solution which is not supported by this API.
 *
 *  Use the function GetTouchScreenCalib() to get a handle to pass on to the other functions. 
 *  Use the method TouchScreenCalib_release(TOUCHSCREENCALIBHANDLE) to return the handle.
 *
 ************************************************/

#ifndef CC_TOUCHSCRCALIB_H
#define CC_TOUCHSCRCALIB_H

#include "CCAuxErrors.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* TOUCHSCREENCALIBHANDLE;

  /*! Touch screen calibration modes */
  typedef enum
  {
    MODE_UNKNOWN = 0,         /*! Unknown mode. */
    MODE_NORMAL = 1,          /*! Normal operation mode. */
    MODE_CALIBRATION_5P = 2,  /*! Calibration with 5 points mode. */
    MODE_CALIBRATION_9P = 3,  /*! Calibration with 9 points mode. */
    MODE_CALIBRATION_13P = 4  /*! Calibration with 13 points mode. */
  } CalibrationModeSettings;

  /*! Touch screen calibration parameters */
  typedef enum
  {
    CONFIG_CALIBRATION_WITH = 0,              /*! Accepted error value when calibrating. */
    CONFIG_CALIBRATION_MEASUREMENTS = 1,      /*! Number of measurements to accept a calibration point. */
    CONFIG_5P_CALIBRATION_POINT_BORDER = 2,	  /*! The number of pixels from the border where the 5 point calibration points should be located. */
    CONFIG_13P_CALIBRATION_POINT_BORDER = 3,  /*! The number of pixels from the border where the 13 point calibration points should be located. */
    CONFIG_13P_CALIBRATION_TRANSITION_MIN = 4,/*! Min defines the transition area in number of pixels, where the two different calibrations are used. */
    CONFIG_13P_CALIBRATION_TRANSITION_MAX = 5	/*! Max defines the transition area in number of pixels, where the two different calibrations are used. */
  } CalibrationConfigParam;

  /*!
   * Factory function that creates instances of the TouchScreenCalib object.
   *
   * Supported Platform(s): XL, XM, XS, XA
   *
   * @return TOUCHSCREENCALIBHANDLE to an allocated TouchScreenCalib object. The returned
   * handle needs to be deallocated using the TouchScreenCalib_release(TOUCHSCREENCALIBHANDLE) 
   * method when it's no longer needed.
   * Returns NULL if it fails to allocate memory.
   */
  EXTERN_C
    CCAUXDLL_API
    TOUCHSCREENCALIBHANDLE
    CCAUXDLL_CALLING_CONV
    GetTouchScreenCalib(void);

  /*!
   * Delete the TouchScreenCalib object.
   *
   * Supported Platform(s): XL, XM, XS, XA
   *
   * @return -
   */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    TouchScreenCalib_release(TOUCHSCREENCALIBHANDLE);

  /*!
   * Set mode of front controller.
   *
   * Supported Platform(s): XL, XM, XS, XA
   *
   * @param mode Selected calibration mode
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    TouchScreenCalib_setMode (TOUCHSCREENCALIBHANDLE, CalibrationModeSettings mode);

  /*!
   * Get mode of front controller.
   *
   * Supported Platform(s): XL, XM, XS, XA
   *
   * @param mode Current calibration mode
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    TouchScreenCalib_getMode (TOUCHSCREENCALIBHANDLE, CalibrationModeSettings* mode);

  /*!
   * Set calibration point
   *
   * Supported Platform(s): XL, XM, XS, XA
   *
   * @param pointNr Calibration point number (1 to total number of points)
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    TouchScreenCalib_setCalibrationPoint (TOUCHSCREENCALIBHANDLE, uint8_t pointNr);

  /*!
   * Check if a calibration point is finished
   *
   * Supported Platform(s): XL, XM, XS, XA
   *
   * @param finished Is current point finished?
   * @param pointNr Calibration point number (1 to total number of points)
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    TouchScreenCalib_checkCalibrationPointFinished (TOUCHSCREENCALIBHANDLE, bool* finished, uint8_t pointNr);

  /*!
   * Get calibration config parameters
   *
   * Supported Platform(s): XL, XM, XS, XA
   *
   * @param param Config parameter
   * @param value Parameter value
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    TouchScreenCalib_getConfigParam (TOUCHSCREENCALIBHANDLE, CalibrationConfigParam param, uint16_t* value);

  /*!
   * Set calibration config parameters
   *
   * Supported Platform(s): XL, XM, XS, XA
   *
   * @param param Config parameter
   * @param value parameter value
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    TouchScreenCalib_setConfigParam (TOUCHSCREENCALIBHANDLE, CalibrationConfigParam param, uint16_t value);

  /*!
  * Perform automatic sensor calibration
  *
  * Supported Platform(s): VA
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    TouchScreenCalib_autoSensorCalib (TOUCHSCREENCALIBHANDLE);


#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_TOUCHSCRCALIB_H */


