/************************************************
 *
 *  CROSSCONTROL AUX API
 *
 *  Video.h
 *
 *  Version 2.15.2.0, 2020-07-08
 *
 *  (c) CrossControl 2010-2020
 *

 *  File description: 
 *  Analog Video
 *
 *  Use the function GetVideo() to get a handle to pass on to the other functions. 
 *  Use the method Video_release(VIDEOHANDLE) to return the handle.
 *
 ************************************************/

#ifndef CC_VIDEO_H
#define CC_VIDEO_H

#include "CCAuxErrors.h"
#include "CCAuxTypes.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* VIDEOHANDLE;

  /*!
   * Factory function that creates instances of the Video object.
   *
   * Supported Platform(s): XL, XM, XS, XA, VC, VA, VS 
   *
   * @return VIDEOHANDLE to an allocated Video object. The returned
   * handle needs to be deallocated using the Video_release(VIDEOHANDLE) 
   * method when it's no longer needed.
   * Returns NULL if it fails to allocate memory.
   */
  EXTERN_C
    CCAUXDLL_API
    VIDEOHANDLE
    CCAUXDLL_CALLING_CONV
    GetVideo(void);

  /*!
   * Delete the Video object.
   *
   * Supported Platform(s): XL, XM, XS, XA, VC, VA, VS
   *
   * @return -
   */
  EXTERN_C
    CCAUXDLL_API
    void 
    CCAUXDLL_CALLING_CONV
    Video_release(VIDEOHANDLE);

  /*!
   * Initialize a video device.
   * The video device will initially use the following settings:
   * DeInterlace_BOB and mirroring disabled.
   *
   * Supported Platform(s): XL, XM, XS, XA, VC, VA, VS
   *
   * @param deviceNr Device to connect to (1,2). Select one of 2 devices to 
   * connect to. (VC platform has only 1 device)
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_init(VIDEOHANDLE, uint8_t deviceNr);

  /*!  
   * Show or hide the video image.
   * Note that it may take some time before the video is shown and correct input info can be read by getRawImage.
   *
   * Supported Platform(s): XL, XM, XS, XA, VC, VA, VS      
   *
   * @param show True shows the video image.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_showVideo(VIDEOHANDLE, bool show);	

  /*!  
   * Set the deinterlace mode used when decoding the interlaced video stream.
   *
   * Supported Platform(s): XL, XM
   *
   * @param mode The mode to set. See enum DeInterlaceMode for descriptions of the modes.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_setDeInterlaceMode(VIDEOHANDLE, DeInterlaceMode mode);

  /*!  
   * Get the deinterlace mode used when decoding the interlaced video stream.
   *
   * Supported Platform(s): XL, XM
   *
   * @param mode The current mode. See enum DeInterlaceMode for descriptions of the modes.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_getDeInterlaceMode(VIDEOHANDLE, DeInterlaceMode* mode);

  /*!  
   * Enable or disable mirroring of the video image.
   *
   * Supported Platform(s): XL, XM, XS, XA, VC, VA, VS      
   *
   * @param mode The mode to set. Enabled or Disabled.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_setMirroring(VIDEOHANDLE, CCStatus mode);

  /*!  
   * Get the current mirroring mode of the video image.
   *
   * Supported Platform(s): XL, XM, XS, XA, VC, VA, VS      
   *
   * @param mode The current mode. Enabled or Disabled.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_getMirroring(VIDEOHANDLE, CCStatus* mode);

  /*!  
   * Set the current rotation of the video image.
   *
   * Supported Platform(s): XA, XS, VC, VA, VS    
   *
   * @param rotation Enum defining the rotation to set.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_setRotation(VIDEOHANDLE, VideoRotation rotation);

  /*!  
   * Get the current rotation of the video image.
   *
   * Supported Platform(s): XA, XS, VC, VA, VS    
   *
   * @param rotation Enum defining the current rotation.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_getRotation(VIDEOHANDLE, VideoRotation *rotation);

  /*!
   * Sets the active video channel.
   *
   * Supported Platform(s): XL, XM, XS, XA, VC, VA, VS      
   *
   * @param channel Enum defining available channels. (VC and VS platforms have only 1 channel, Analog_Channel_1)
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_setActiveChannel(VIDEOHANDLE, VideoChannel channel);

  /*!
   * Get the current video channel.
   *
   * Supported Platform(s): XL, XM, XS, XA, VC, VA
   *
   * @param channel Enum defining available channels. (VC platform has only 1 channel, Analog_Channel_1)
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_getActiveChannel(VIDEOHANDLE, VideoChannel *channel);

  /*!
   * Set color keys. Writes RGB color key values.
   * Note that the system uses 18 bit colors, so the two least significant bits 
   * are not used.
   *
   * Supported Platform(s): XL, XM
   *
   * @param rKey Red key value.
   * @param gKey Green key value.
   * @param bKey Blue key value.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_setColorKeys(VIDEOHANDLE, uint8_t rKey, uint8_t gKey, uint8_t bKey);

  /*!
   * Get color key values. Note that the system uses 18 bit colors, so the two 
   * least significant bits are not used.
   *
   * Supported Platform(s): XL, XM
   *
   * @param rKey Red value.
   * @param gKey Green value.
   * @param bKey Blue value.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_getColorKeys(VIDEOHANDLE, uint8_t *rKey, uint8_t *gKey, uint8_t *bKey);

  /*!
   * Set the area where video is shown.
   *
   * Supported Platform(s): XL, XM, XS, XA, VC, VA, VS     
   *
   * @param topLeftX Top left X coordinate on screen.
   * @param topLeftY Top left Y coordinate on screen.
   * @param bottomRightX Bottom right X coordinate on screen.
   * @param bottomRightY Bottom right Y coordinate on screen.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_setVideoArea(VIDEOHANDLE, uint16_t topLeftX, uint16_t topLeftY, uint16_t bottomRightX, uint16_t bottomRightY);

  /*!
   * Get the raw image size of moving image before any scaling and frame rate.
   * For snapshot the height is 4 row less.
   *
   * Supported Platform(s): XL, XM, XS, XA, VC, VA, VS    
   *
   * @param width Width of raw image.
   * @param height Height of raw moving image, snapshot are 4 bytes less.
   * @param frameRate Received video frame rate.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_getRawImage(VIDEOHANDLE, uint16_t *width, uint16_t *height, float32_t *frameRate);

  /*!
   * Get the area where video is shown.
   *
   * Supported Platform(s): XL, XM, XS, XA, VC, VA, VS    
   *
   * @param topLeftX Top left X coordinate on screen.
   * @param topLeftY Top left Y coordinate on screen.
   * @param bottomRigthX Bottom right X coordinate on screen.
   * @param bottomRigthY Bottom right Y coordinate on screen.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_getVideoArea(VIDEOHANDLE, uint16_t *topLeftX, uint16_t *topLeftY, uint16_t *bottomRigthX, uint16_t *bottomRigthY);

  /*!
   * Get video standard.
   * The video decoder auto detects the video standard of the source.
   *
   * Supported Platform(s): XL, XM, XS, XA, VC, VA, VS   
   *
   * @param standard Video standard.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_getVideoStandard(VIDEOHANDLE, videoStandard *standard);

  /*!
   * Video status byte.
   *
   * Supported Platform(s): XL, XM, XS, XA, VC, VA, VS  
   *
   * @param status Status byte
   *        Bit 0: video on/off 0 = Off, 1 = On.
   *        Bit 2-1: De-interlacing method, 
   *                 0 = Only even rows, 
   *                 1 = Only odd rows, 
   *                 2 = BOB, 
   *                 3 = invalid.
   *        Bit 3: Mirroring mode, 0 = Off, 1 = On
   *        Bit 4: Read or write operation to analogue video decoder in 
   *               progress.
   *        Bit 5: Analogue video decoder ready bit.
   *
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_getStatus(VIDEOHANDLE, uint8_t *status);

  /*!
   * Set Video Scaling (image size).
   * If the deinterlace mode is set to DeInterlace_Even or DeInterlace_Odd, this function multiplies the
   * vertical scaling by a factor of two, to get the correct image proportions.
   *
   * Supported Platform(s): XL, XM
   *
   * @param x Horizontal scaling (0.25-4).
   * @param y Vertical scaling (0.25-4 DeInterlace_BOB) (0.125-2 DeInterlace_Even, DeInterlace_Odd).
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_setScaling(VIDEOHANDLE, float32_t x, float32_t y);

  /*!
   * Get Video Scaling (image size).
   * If the deinterlace mode is set to DeInterlace_Even or DeInterlace_Odd, this function divides the
   * actual vertical scaling by a factor of two, to get the same scaling factor as set with setScaling.
   *
   * Supported Platform(s): XL, XM
   *
   * @param x Horizontal scaling (0.25-4).
   * @param y Vertical scaling (0.25-4 DeInterlace_BOB) (0.125-2 DeInterlace_Even, DeInterlace_Odd).
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_getScaling(VIDEOHANDLE, float32_t *x, float32_t *y);

  /*!
   * To be able to take snapshot the snapshot function has to be active.
   * After activation it takes 120ms before first snapshot can be
   * taken. The Snapshot function can be active all the time. If power consumption
   * and heat is an issue, snapshot may be turned off.
   *
   * Supported Platform(s): XL, XM (Windows)  
   *
   * @param activate Set to true if the snapshot function shall be active.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_activateSnapshot(VIDEOHANDLE, bool activate);

  /*!
   * Takes a snapshot of the current video image and stores it to a bitmap file.
   * This is a combination of takeSnapShotRaw, getVideoStandard and createBitMap and
   * then storing of the bmpBuffer to file.
   * To be able to take a snapshot, the snapshot function has to be active.
   *
   * Supported Platform(s): XL, XM (Windows)  
   *
   * @param path The file path to where the image should be stored.
   * @param bInterlaced If true the bitmap only contains every second line in the image, to save bandwidth.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_takeSnapshot(VIDEOHANDLE, const char_t *path, bool bInterlaced);

  /*!
   * Takes a snapshot of the current video image and return raw image data.
   * The size of the raw image is when interlaced = false
   *     0x100 + line count * row count * 4.
   * The size of the raw image is when interlaced = true
   *     0x100 + line count * row count * 2.
   * To be able to take a snapshot, the snapshot function has to be active.
   * This function is blocking until a new frame is available from the decoder.
   * An error will be returned if the decoder doesn't return any frames before a timeout.
   *
   * Supported Platform(s): XL, XM (Windows)   
   *
   * @param rawImgBuffer Buffer for image to be stored in.
   * @param rawImgBuffSize Size of the buffer.
   * @param bInterlaced If true the bitmap only contains every second
   * line in the image, to save bandwidth.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_takeSnapshotRaw(VIDEOHANDLE, char_t *rawImgBuffer, uint32_t rawImgBuffSize, bool bInterlaced);

  /*!
   * Takes a snapshot of the current video image and return a data buffer with a
   * bitmap image.
   * The bmp buffer is allocated in the function and has to be deallocated with
   * freeBmpBuffer() by the application.
   * This is a combination of the function takeSnapShotRaw and createBitMap.
   * To be able to take a snapshot, the snapshot function has to be active.
   *
   * Supported Platform(s): XL, XM (Windows)  
   * 
   * @param bmpBuffer Bitmap ram buffer allocated by the API, has to be 
   * deallocated with freeBmpBuffer() by the application.
   * @param bmpBufSize Size of the returned bitmap buffer.
   * @param bInterlaced If true the bitmap only contains every second
   * line in the image, to save bandwidth.
   * @param bNTSCFormat True if the video format in rawImageBuffer is NTSC format.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_takeSnapshotBmp(VIDEOHANDLE, char_t **bmpBuffer, uint32_t *bmpBufSize, bool bInterlaced, bool bNTSCFormat);

  /*!
   * Create a bitmap from a raw image buffer.
   * The bmp buffer is allocated in the function and has to be 
   * deallocated by the application.
   *
   * Supported Platform(s): XL, XM (Windows)  
   *
   * @param bmpBuffer Bitmap ram buffer allocated by the API, has to be
   * deallocated with freeBmpBuffer() by the application.
   * @param bmpBufSize Size of the returned bitmap buffer.
   * @param rawImgBuffer Raw image buffer from takeSnapShotRaw.
   * @param rawImgBufSize Size of the raw image buffer.
   * @param bInterlaced Interlaced, if true the bitmap only contains every
   * second line in the image, to save bandwidth.
   * @param bNTSCFormat True if the video format in rawImageBuffer is NTSC format.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_createBitmap(VIDEOHANDLE,
		       char_t **bmpBuffer,
		       uint32_t *bmpBufSize,
		       const char_t* rawImgBuffer,
		       uint32_t rawImgBufSize,
		       bool bInterlaced,
		       bool bNTSCFormat);

  /*!
   * Free the memory allocated for BMP buffer.
   *
   * Supported Platform(s): XL, XM (Windows)
   *
   * @param bmpBuffer The bmp buffer to free.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_freeBmpBuffer(VIDEOHANDLE, char_t *bmpBuffer);

  /*!
   * Minimizes the video area.
   * Restore with restore() call.
   *
   * Supported Platform(s): XL, XM, XS, XA, VC, VA, VS   
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_minimize(VIDEOHANDLE);

  /*!
   * Restores the video area to the size it was before a minimize() call.
   * Don't use restore if minimize has not been used first.
   *
   * Supported Platform(s): XL, XM, XS, XA, VC, VA, VS    
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_restore(VIDEOHANDLE);

  /*!
   * Set Video decoder bus register.
   * Advanced function for direct access to the video decoder TVP5150AM1
   * registers.
   *
   * Supported Platform(s): XL, XM, XS, XA, VC, VA
   *
   * @param decoderRegister Decoder Register Address.
   * @param registerValue register value.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_setDecoderReg(VIDEOHANDLE, uint8_t decoderRegister, uint8_t registerValue);

  /*!
   * Get Video decoder bus register.
   * Advanced function for direct access to the video decoder TVP5150AM1
   * registers.  
   *
   * Supported Platform(s): XL, XM, XS, XA, VC, VA
   *
   * @param decoderRegister Decoder Register Address.
   * @param registerValue register value.
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_getDecoderReg(VIDEOHANDLE, uint8_t decoderRegister, uint8_t *registerValue);

  /*!
   * Crop video image.
   * Note that the video chip manual says the following about horisontal cropping:
   * The number of pixels of active video must be an even number.
   * The parameters top and bottom are internally converted to an even number. This is due to the input video being interlaced,
   * a pair of odd/even lines are allways cropped together.
   * On XA/XS platforms, cropping from top/bottom on device 2 (channels 3 and 4) is not supported.
   *
   * Supported Platform(s): XL, XM, XS, XA, VC, VA
   * 
   * @param top Crop top (0-255 lines).
   * @param left Crop left (0-127 lines).
   * @param bottom Crop bottom (0-255 lines).
   * @param right Crop right (0-127 lines).
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_setCropping(VIDEOHANDLE, uint8_t top, uint8_t left, uint8_t bottom, uint8_t right);

  /*!
   * Get Crop parameters.
   *
   * Supported Platform(s): XL, XM, XS, XA, VC, VA
   *
   * @param top Crop top (lines).
   * @param left Crop left (lines).
   * @param bottom Crop bottom (lines).
   * @param right Crop right (lines).
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_getCropping(VIDEOHANDLE, uint8_t *top, uint8_t *left, uint8_t *bottom, uint8_t *right);

  /*!
   * Copy one frame from camera to the display.
   *
   * Supported Platform(s): XA, XS, VC, VA
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_showFrame(VIDEOHANDLE);

  /*!
   * Enable or disable overlaying of graphics on top of video.
   *
   * Supported Platform(s): XA, XS, VC, VA
   *
   * @param mode Overlay enable mode
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_setGraphicsOverlay(VIDEOHANDLE, CCStatus mode);

  /*!
   * Get the current graphics overlaying mode.
   *
   * Supported Platform(s): XA, XS, VC, VA
   *
   * @param mode Overlay enable mode
   *
   * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
   */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Video_getGraphicsOverlay(VIDEOHANDLE, CCStatus* mode);

#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_VIDEO_H */

